﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using Discovery.Properties;
using Tobii.Eyetracking.Sdk;

namespace Discovery
{
    /*
     * This is a base class in the Discovery application. All demo applications that needs eye interaction
     * subclasses this class, so it's used quite heavily. It's main purpose is to collect common functions used
     * by other classes and to retrieve raw eye gaze x and y data for both left and right eye, lowpass them and
     * update the gaze dot location. In this way each demo applicaiton does not have to do this themselves, but
     * can focus on their own specific implementation that uses the lowpassed eye gaze values.
     * 
     * Haakon Raudsandmoen and Boerge Roedsjoe, Trondheim, 16. February 2012
     */
    public partial class BaseForm : Form
    {
        // These two ques are used by the low pass filter to smoothen out the eye gaze
        internal readonly FixedSizedQueue<int> _gazeDataX;
        internal readonly FixedSizedQueue<int> _gazeDataY;

        // These two are used for the Calibration Test demo application
        internal readonly FixedSizedQueue<int> _gazeDataXLowPassedHistory;
        internal readonly FixedSizedQueue<int> _gazeDataYLowPassedHistory;

        // The current x and y eye gaze lowpassed
        internal int _eyeGazeXLowPassed;
        internal int _eyeGazeYLowPassed;

        internal Size GAZE_DOT_SIZE = new Size(8, 8);

        internal readonly TimeSpan DWELL_TIME = new TimeSpan(0, 0, 0, 1, 500);
        internal DateTime _timeWhenUserStartedLooking;

        internal const double SCROLLABLE_AREA_PERCENT = 0.15;
        internal const double SCROLLABLE_READ_AREA_PEPRCENT = 0.10;

        internal GazeForm GazeDot { get; set; }

        internal readonly List<Color> _colors;
        internal readonly List<Color> _minicolors;
        internal int _colorIndex;
        internal Timer _timer;
        internal TreeNode tempNode;

        internal bool HoverEffectIsEnabled { get; set; }
        internal bool FirstTimeUserLooksAtControl { get; set; }
        internal bool FirstTimeUserLooksAtNode { get; set; }

        // Constructor that initializes the design and sets the internal properties
        public BaseForm()
        {
            InitializeComponent();

            GazeDot = GazeForm.Instance;

            _timer = new Timer();
            _timer.Tick += HandleTimerTick;

            _minicolors = new List<Color>
                          {
                              Color.FromArgb(100, 255, 0, 0),
                              Color.FromArgb(100, 0, 255, 0),
                          };

            _colors = new List<Color>
                          {
                              Color.FromArgb(100, 255, 0, 0),
                              Color.FromArgb(100, 0, 255, 0),
                              Color.FromArgb(100, 0, 0, 255),
                              Color.FromArgb(100, 0, 0, 0),
                              Color.FromArgb(100, 255, 0, 255),
                              Color.FromArgb(100, 255, 255, 0), 
                              Color.FromArgb(100, 0, 255, 255),
                          };
            _colorIndex = 0;

            _gazeDataX = new FixedSizedQueue<int>(7);
            _gazeDataY = new FixedSizedQueue<int>(7);

            _gazeDataXLowPassedHistory = new FixedSizedQueue<int>(7);
            _gazeDataYLowPassedHistory = new FixedSizedQueue<int>(7);

            _timeWhenUserStartedLooking = DateTime.Now;
            
            FirstTimeUserLooksAtControl = true;
            HoverEffectIsEnabled = true;

            _dwellTimeLabel.BackColor = DefaultBackColor;

            KeyPreview = true;
            KeyDown += GazeInteraction_KeyDown;
        }

        // sets the timer interval in milliseconds. This is used for count down timers
        internal void SetTimerInterval(int milliseconds)
        {
            _timer.Interval = milliseconds;
        }

        // Stops the Timer and performs the given action 
        internal void StopTimer(Action action)
        {
            if (action != null)
                Invoke(action);
            if (_timer.Enabled)
                _timer.Stop();
        }

        // Starts the Timer and performs the given action
        internal void StartTimer(Action action)
        {
            if (!_timer.Enabled)
            {
                if (action != null)
                    Invoke(action);
                _timer.Start();
            }
        }

        // Virtual method that should be override'd by child classes to handle timer tick
        internal virtual void HandleTimerTick(object sender, EventArgs e)
        {
        }

        // Takes the raw gaze data x and y for both left and right eye and sends them through a low pass filter. 
        // Saving a history of both the raw gaze data and lowpassed data. Updates the gaze dot location.
        internal virtual void GazeDataReceived(object sender, GazeDataEventArgs e)
        {
            var gd = e.GazeDataItem;

            UpdateValidBarColor(GazeDataValid(gd));

            var currentX = (int)(((gd.LeftGazePoint2D.X + gd.RightGazePoint2D.X) / 2) * Screen.PrimaryScreen.Bounds.Width);
            var currentY = (int)(((gd.LeftGazePoint2D.Y + gd.RightGazePoint2D.Y) / 2) * Screen.PrimaryScreen.Bounds.Height);

            // Storing raw eye gaze positions
            _gazeDataX.Enqueue(currentX);
            _gazeDataY.Enqueue(currentY);

            _eyeGazeXLowPassed = LowPass(_gazeDataX.Queue.ToArray(), 1, 1);
            _eyeGazeYLowPassed = LowPass(_gazeDataY.Queue.ToArray(), 1, 1);

            // Storing lowpassed eye gaze positions
            _gazeDataXLowPassedHistory.Enqueue(_eyeGazeXLowPassed);
            _gazeDataYLowPassedHistory.Enqueue(_eyeGazeYLowPassed);

            UpdateGazeFormLocation(_eyeGazeXLowPassed, _eyeGazeYLowPassed);
        }

        // Shows and hides the gaze marker with "Ctrl+D" keyboard shortcut
        internal void GazeInteraction_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control && e.KeyCode == Keys.H)
                HoverEffectIsEnabled = !HoverEffectIsEnabled;

            if (e.Control && e.KeyCode == Keys.D)
                _dwellTimeLabel.Visible = !_dwellTimeLabel.Visible;

            if (e.Control && e.KeyCode == Keys.G)
            {
                try
                {
                    if (!GazeDot.Visible)
                    {
                        GazeDot.Show();
                        GazeDot.Size = GAZE_DOT_SIZE;
                        GazeDot.Visible = true;
                    }
                    else
                    {
                        GazeDot.Visible = false;
                        GazeDot.Size = GAZE_DOT_SIZE;
                        GazeDot.Hide();
                    }
                }
                catch (ObjectDisposedException)
                {
                    MessageBox.Show("Do not close the gaze dot form manually, only use " +
                                    "Ctrl+D to hide it, or go to the Calibration windows " +
                                    "and click the \"Hide Gaze Dot\" button. \n\n Please restart the application " +
                                    "to be able to show the gaze dot again", "Use Ctrl+D to close the gaze dot",
                                    MessageBoxButtons.OK,
                                    MessageBoxIcon.Error);
                }
            }
        }

        // Code that displays the gaze dot exactly where the user looks
        internal void UpdateGazeFormLocation(int gazeDataXLowPassed, int gazeDataYLowPassed)
        {
            GazeDot.Left = gazeDataXLowPassed - (GazeDot.Width / 2);
            GazeDot.Top = gazeDataYLowPassed - (GazeDot.Height / 2);
        }

        // Eye gaze data is valid if bouth eyes are found in the eye tracker camera
        internal static bool GazeDataValid(IGazeDataItem gd)
        {
            return gd.LeftValidity == 0 && gd.RightValidity == 0;
        }

        // Determines if the user is gazing at a Windows Control
        internal bool IsUserGazingAt(Control control)
        {
            try
            {
                return GazeDot.Location.X > control.Parent.PointToScreen(control.Location).X &&
                GazeDot.Location.X < control.Parent.PointToScreen(control.Location).X + control.Width &&
                GazeDot.Location.Y > control.Parent.PointToScreen(control.Location).Y &&
                GazeDot.Location.Y < control.Parent.PointToScreen(control.Location).Y + control.Height;
            }
            catch (Exception ee)
            {
                //MessageBox.Show("Something weird happened in the IsUserGazingAt method. Something to do with button's parent? \n\n" + ee,
                //    "Weird error", MessageBoxButtons.OK,
                //        MessageBoxIcon.Error);
                return false;
            }
        }
        private void UpdateValidBarColor(bool gazeDataValid)
        {
            _gazeDataValidBar.BackColor = gazeDataValid ? Color.Green : Color.Red;
        }

        // Determines if the user is looking at a scroll-activated area.
        internal bool UserWantsToScroll(Control control, string direction)
        {
            var userWantsToScroll = false;

            if (direction == "down")
                userWantsToScroll = GazeDot.Location.X > control.Parent.PointToScreen(control.Location).X &&
                       GazeDot.Location.X < control.Parent.PointToScreen(control.Location).X + control.Width &&
                       GazeDot.Location.Y > control.Parent.PointToScreen(control.Location).Y + control.Height * (1 - SCROLLABLE_AREA_PERCENT) &&
                       GazeDot.Location.Y < control.Parent.PointToScreen(control.Location).Y + control.Height;
            if (direction == "up")
                userWantsToScroll = GazeDot.Location.X > control.Parent.PointToScreen(control.Location).X &&
                       GazeDot.Location.X < control.Parent.PointToScreen(control.Location).X + control.Width &&
                       GazeDot.Location.Y > control.Parent.PointToScreen(control.Location).Y &&
                       GazeDot.Location.Y < control.Parent.PointToScreen(control.Location).Y + control.Height * SCROLLABLE_AREA_PERCENT;
            if (direction == "right")
                userWantsToScroll = GazeDot.Location.X > control.Parent.PointToScreen(control.Location).X + control.Width * (1 - SCROLLABLE_AREA_PERCENT) &&
                       GazeDot.Location.X< control.Parent.PointToScreen(control.Location).X + control.Width &&
                       GazeDot.Location.Y > control.Parent.PointToScreen(control.Location).Y &&
                       GazeDot.Location.Y < control.Parent.PointToScreen(control.Location).Y + control.Height;
            if (direction == "left")
                userWantsToScroll = GazeDot.Location.X > control.Parent.PointToScreen(control.Location).X &&
                       GazeDot.Location.X< control.Parent.PointToScreen(control.Location).X + control.Width * SCROLLABLE_AREA_PERCENT &&
                       GazeDot.Location.Y > control.Parent.PointToScreen(control.Location).Y &&
                       GazeDot.Location.Y < control.Parent.PointToScreen(control.Location).Y + control.Height;
            return userWantsToScroll;
        }

        // Determines if the user has looked long enough a a control
        internal bool UserHasLookedLongEnough()
        {
            if (FirstTimeUserLooksAtControl)
            {
                _timeWhenUserStartedLooking = DateTime.Now;
                FirstTimeUserLooksAtControl = false;
            }

            if (DateTime.Now - _timeWhenUserStartedLooking > DWELL_TIME)
            {
                FirstTimeUserLooksAtControl = true;
                return true;
            }
            return false;
        }

        // Determines if the user has looked long enough a a control
        internal bool UserHasLookedLongEnough(TimeSpan dwellTime)
        {
            if (FirstTimeUserLooksAtControl)
            {
                _timeWhenUserStartedLooking = DateTime.Now;
                FirstTimeUserLooksAtControl = false;
            }

            if (DateTime.Now - _timeWhenUserStartedLooking > dwellTime)
            {
                FirstTimeUserLooksAtControl = true;
                return true;
            }
            return false;
        }

        // Determines if the user has stared for amount of time given by the TimeSpan argumen
        internal bool UserHasLookedLongEnoughAtNode(TimeSpan dwellTime, TreeNode node)
        {
            if (FirstTimeUserLooksAtNode)
            {
                _timeWhenUserStartedLooking = DateTime.Now;
                FirstTimeUserLooksAtNode = false;
                tempNode = node;
            }
            if (tempNode == node)
            {
                if (DateTime.Now - _timeWhenUserStartedLooking > dwellTime)
                {
                    FirstTimeUserLooksAtNode = true;
                    return true;
                }
            }
            else 
                FirstTimeUserLooksAtNode = true;
            
            return false;
        }

        // Performs button interaction on the button argument. That is making a hover effect and performing
        // a click
        internal void DoButtonInteraction(Button button)
        {
            if (FirstTimeUserLooksAtControl)
            {
                _timeWhenUserStartedLooking = DateTime.Now;
                FirstTimeUserLooksAtControl = false;
                if (HoverEffectIsEnabled)
                    button.BackgroundImage = Resources.eye_hover;
            }

            else if (DateTime.Now - _timeWhenUserStartedLooking > DWELL_TIME)
            {
                button.PerformClick();
                if (HoverEffectIsEnabled)
                    button.BackgroundImage = null;
                FirstTimeUserLooksAtControl = true;
            }
        }

        internal void DoButtonInteraction(Button button, TimeSpan dwellTime)
        {
            if (FirstTimeUserLooksAtControl)
            {
                _timeWhenUserStartedLooking = DateTime.Now;
                FirstTimeUserLooksAtControl = false;
                if (HoverEffectIsEnabled)
                    button.BackgroundImage = Resources.eye_hover;
            }

            else if (DateTime.Now - _timeWhenUserStartedLooking > dwellTime)
            {
                button.PerformClick();
                if (HoverEffectIsEnabled)
                    button.BackgroundImage = null;
                FirstTimeUserLooksAtControl = true;
            }
        }

        internal void DoButtonInteractionForColorButtons(Button button, TimeSpan dwellTime)
        {
            if (FirstTimeUserLooksAtControl)
            {
                _timeWhenUserStartedLooking = DateTime.Now;
                FirstTimeUserLooksAtControl = false;
            }

            else if (DateTime.Now - _timeWhenUserStartedLooking > dwellTime)
            {
                button.PerformClick();
                button.BackgroundImage = null;
                FirstTimeUserLooksAtControl = true;
            }
        }

        // Resets the buttons to it's default properties
        internal void RemoveHoverEffectFromButtonsOtherThanMe(List<Button> buttons, Button me)
        {
            foreach (var button in buttons)
            {
                if (button == me)
                    continue;
                button.BackgroundImage = null;
            }
        }

        // Resets the buttons to it's default properties
        internal void RemoveHoverEffectFromButtons(List<Button> buttons)
        {
            foreach (var button in buttons)
            {
                button.BackgroundImage = null;
            }
        }

        // Return rc low-pass filter output samples, given input samples,
        // time interval dt, and time constant rc
        internal static int LowPass(IList<int> x, double dt, double rc)
        {
            var y = new int[x.Count];
            var a = dt / (rc + dt);
            y[0] = x[0];
            for (var i = 1; i < x.Count; i++)
            {
                y[i] = (int)(a * x[i] + (1 - a) * y[i - 1]);
            }
            return y[y.Length - 1];
        }

        internal TimeSpan IncreaseDwellTime(TimeSpan dwellTime)
        {
            dwellTime += new TimeSpan(0, 0, 0, 0, 250);
            if (dwellTime >= new TimeSpan(0, 0, 0, 5))
                dwellTime = new TimeSpan(0, 0, 0, 5);
            return dwellTime;
        }
        
        internal TimeSpan DecreaseDwellTime(TimeSpan dwellTime)
        {
            dwellTime -= new TimeSpan(0, 0, 0, 0, 250);
            if (dwellTime <= TimeSpan.Zero)
                dwellTime = new TimeSpan(0, 0, 0, 0, 250);
            return dwellTime;
        }

        internal void UpdateDwellTimeLabel(TimeSpan t)
        {
            _dwellTimeLabel.Text = "Dwell Time: " + t.TotalMilliseconds + " ms";
        }
    }
}