﻿using System;
using System.Collections.Generic;
using System.Reflection;
using System.Windows.Forms;
using Discovery.Properties;

namespace Discovery.Browser
{
    public partial class WebBrowserForm : BaseForm
    {
        private readonly HomeForm _homeForm;
        private int _webBrowserHorizontalScroll;
        private int _webBrowserVerticalScroll;
        private int _shouldScrollOrNotCounter;
        private readonly List<Button> _buttons;

        private static TimeSpan DWELL_TIME_BROWSER_BUTTONS = new TimeSpan(0, 0, 0, 1, 500);
        private static int DEFAULT_START_SCROLL_SPEED = 4;

        private int _scrollSpeed = DEFAULT_START_SCROLL_SPEED;
        private const int SCROLL_SPEED_MAX = 10;

        public WebBrowserForm(HomeForm homeForm)
        {
            InitializeComponent();

            _exitApplicationButton.Visible = false;

            _homeForm = homeForm;
            SetTimerInterval(500); // For every timer tick the scroll speed increments when user is actively scrolling

            _buttons = new List<Button>{
                _homeButton, 
                _goBackButton, 
                _goForwardButton, 
                _ntnuButton, 
                _wikipediaButton};

            KeyPreview = true;
            KeyDown += WebBrowserForm_KeyDown;
        }

        private void WebBrowserForm_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control && e.KeyCode == Keys.Add)
            {
                DWELL_TIME_BROWSER_BUTTONS = IncreaseDwellTime(DWELL_TIME_BROWSER_BUTTONS);
                Log.Write("Browser Application: Dwell time: " + DWELL_TIME_BROWSER_BUTTONS.TotalMilliseconds + " ms");
            }
            if (e.Control && e.KeyCode == Keys.Subtract)
            {
                DWELL_TIME_BROWSER_BUTTONS = DecreaseDwellTime(DWELL_TIME_BROWSER_BUTTONS);
                Log.Write("Browser Application: Dwell time: " + DWELL_TIME_BROWSER_BUTTONS.TotalMilliseconds + " ms");
            }
                
            UpdateDwellTimeLabel(DWELL_TIME_BROWSER_BUTTONS);
        }

        internal override void GazeDataReceived(object sender, Tobii.Eyetracking.Sdk.GazeDataEventArgs e)
        {
            base.GazeDataReceived(sender, e);

            var gd = e.GazeDataItem;

            if (_webBrowser.ReadyState == WebBrowserReadyState.Complete)
            {

                if (GazeDataValid(gd))
                {
                    if (IsUserGazingAt(_homeButton))
                    {
                        RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _homeButton);
                        _homeButton.BackgroundImage = Resources.eye_hover;
                        DoButtonInteraction(_homeButton, DWELL_TIME_BROWSER_BUTTONS);
                    }
                        
                    else if (IsUserGazingAt(_goBackButton))
                    {
                        RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _goBackButton);
                        _goBackButton.BackgroundImage = Resources.eye_hover;
                        DoButtonInteraction(_goBackButton, DWELL_TIME_BROWSER_BUTTONS);
                    }

                    else if (IsUserGazingAt(_goForwardButton))
                    {
                        RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _goForwardButton);
                        _goForwardButton.BackgroundImage = Resources.eye_hover;
                        DoButtonInteraction(_goForwardButton, DWELL_TIME_BROWSER_BUTTONS);
                    }

                    else if (IsUserGazingAt(_ntnuButton))
                    {
                        RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _ntnuButton);
                        _ntnuButton.BackgroundImage = Resources.eye_hover;
                        DoButtonInteraction(_ntnuButton, DWELL_TIME_BROWSER_BUTTONS);
                    }

                    else if (IsUserGazingAt(_wikipediaButton))
                    {
                        RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _wikipediaButton);
                        _wikipediaButton.BackgroundImage = Resources.eye_hover;
                        DoButtonInteraction(_wikipediaButton, DWELL_TIME_BROWSER_BUTTONS);
                    }

                    else if (UserLooksAtReadScrollArea())
                    {
                        if (_shouldScrollOrNotCounter > 2)
                        {
                            ScrollBrowser("down", 1);
                            _shouldScrollOrNotCounter = 0;
                        }
                        else
                            _shouldScrollOrNotCounter++;
                    }

                    else if (UserWantsToScroll(_webBrowser, "down"))
                    {
                        StartTimer(() => _scrollSpeed = DEFAULT_START_SCROLL_SPEED);
                        ScrollBrowser("down", _scrollSpeed);
                    }

                    else if (UserWantsToScroll(_webBrowser, "up"))
                    {
                        StartTimer(() => _scrollSpeed = DEFAULT_START_SCROLL_SPEED);
                        ScrollBrowser("up", _scrollSpeed);
                    }

                    else if (UserWantsToScroll(_webBrowser, "right"))
                    {
                        StartTimer(() => _scrollSpeed = DEFAULT_START_SCROLL_SPEED);
                        ScrollBrowser("right", _scrollSpeed);
                    }

                    else if (UserWantsToScroll(_webBrowser, "left"))
                    {
                        StartTimer(() => _scrollSpeed = DEFAULT_START_SCROLL_SPEED);
                        ScrollBrowser("left", _scrollSpeed);
                    }
                    else
                    {
                        StopTimer(null);
                        FirstTimeUserLooksAtControl = true;
                        RemoveHoverEffectFromButtons(_buttons);
                    }
                }    
            }
        }

        private bool UserLooksAtReadScrollArea()
        {
            var browserWidth = _webBrowser.Parent.PointToScreen(_webBrowser.Location).X + _webBrowser.Width;
            var browserHeight = _webBrowser.Parent.PointToScreen(_webBrowser.Location).Y + _webBrowser.Height;

            return GazeDot.Location.X > _webBrowser.Parent.PointToScreen(_webBrowser.Location).X &&
                       GazeDot.Location.X < browserWidth &&
                       GazeDot.Location.Y > browserHeight * (1 - SCROLLABLE_AREA_PERCENT - SCROLLABLE_READ_AREA_PEPRCENT) &&
                       GazeDot.Location.Y < browserHeight * (1 - SCROLLABLE_AREA_PERCENT);
        }

        private void UpdateBackAndForwardButtons()
        {
            _goForwardButton.Enabled = _webBrowser.CanGoForward;
            _goBackButton.Enabled = _webBrowser.CanGoBack;
        }

        internal override void HandleTimerTick(object sender, EventArgs e)
        {
            if (_scrollSpeed >= SCROLL_SPEED_MAX)
                return;
            ++_scrollSpeed;
        }

        private void ScrollBrowser(string downUpLeftOrRight, int speed)
        {
            if (_webBrowser.Document == null) return;

            var domDocument = _webBrowser.Document.DomDocument;

            var doc = domDocument.GetType().InvokeMember("documentElement", BindingFlags.GetProperty, null,
                                                            domDocument,
                                                            null);
            if (downUpLeftOrRight == "down")
                doc.GetType().InvokeMember("scrollTop", BindingFlags.SetProperty, null, doc,
                                           new object[] { _webBrowserVerticalScroll += speed });
            if (downUpLeftOrRight == "up" && _webBrowserVerticalScroll >= speed)
                doc.GetType().InvokeMember("scrollTop", BindingFlags.SetProperty, null, doc,
                                           new object[] { _webBrowserVerticalScroll -= speed });
            if (downUpLeftOrRight == "left" && _webBrowserHorizontalScroll >= speed)
                doc.GetType().InvokeMember("scrollLeft", BindingFlags.SetProperty, null, doc,
                                           new object[] { _webBrowserHorizontalScroll -= speed });
            if (downUpLeftOrRight == "right")
                doc.GetType().InvokeMember("scrollLeft", BindingFlags.SetProperty, null, doc,
                                           new object[] { _webBrowserHorizontalScroll += speed });
        }

        private void _goForwardButton_Click(object sender, EventArgs e)
        {
            _webBrowser.GoForward();
        }

        private void _goBackButton_Click(object sender, EventArgs e)
        {
            _webBrowser.GoBack();
        }

        private void _homeButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void _ntnuButton_Click(object sender, EventArgs e)
        {
            _webBrowser.Navigate("http://www.ntnu.edu");
        }

        private void _wikipediaButton_Click(object sender, EventArgs e)
        {
            _webBrowser.Navigate("http://en.wikipedia.org/wiki/Eye_tracking");
        }

        private void WebBrowserForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (_homeForm.ConnectedTracker != null)
            {
                _homeForm.ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                _homeForm.ConnectedTracker.GazeDataReceived += _homeForm.GazeDataReceived;
            }
            _homeForm.Show();
        }

        private void _webBrowser_Navigated(object sender, WebBrowserNavigatedEventArgs e)
        {
            UpdateBackAndForwardButtons();
            _scrollSpeed = DEFAULT_START_SCROLL_SPEED;
        }

        private void WebBrowserForm_Load(object sender, EventArgs e)
        {
            UpdateDwellTimeLabel(DWELL_TIME_BROWSER_BUTTONS);
        }
    }
}