﻿using System;
using System.IO;
using System.Windows.Forms;
using Discovery.Calibration_Test;
using Tobii.Eyetracking.Sdk;
using Tobii.Eyetracking.Sdk.Exceptions;

namespace Discovery.Calib
{
    public partial class CalibrationForm : BaseForm
    {
        
        private readonly HomeForm _homeForm;
        internal IEyetracker ConnectedTracker { get; set; }
        
        internal bool CalibrationIsOk { get; set; }
        private static TimeSpan DWELL_TIME_CALIBRATION_BUTTONS = new TimeSpan(0, 0, 0, 1, 500);

        public CalibrationForm(HomeForm homeForm)
        {
            InitializeComponent();
            _homeForm = homeForm;
            ConnectedTracker = _homeForm.ConnectedTracker;

            KeyPreview = true;
            KeyDown += CalibrationForm_KeyDown;
        }

        private void CalibrationForm_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control && e.KeyCode == Keys.Add)
            {
                Log.Write("The Calibration Application: Dwell time: " + DWELL_TIME_CALIBRATION_BUTTONS.TotalMilliseconds + " ms");
                DWELL_TIME_CALIBRATION_BUTTONS = IncreaseDwellTime(DWELL_TIME_CALIBRATION_BUTTONS);
            }
            if (e.Control && e.KeyCode == Keys.Subtract)
            {
                Log.Write("The Calibration Application: Dwell time: " + DWELL_TIME_CALIBRATION_BUTTONS.TotalMilliseconds + " ms");
                DWELL_TIME_CALIBRATION_BUTTONS = DecreaseDwellTime(DWELL_TIME_CALIBRATION_BUTTONS);
            }

            UpdateDwellTimeLabel(DWELL_TIME_CALIBRATION_BUTTONS);
        }

        private void UpdateUiElements()
        {
            // Some different states the GUI elements can respons to in this form:
            // No eye tracker found : all buttons disabled
            // Eye tracker found : load calibration button and view calibration button enabled
            // Eye tracker found and tracking : all buttons enabled except save calibration button
            // Calibration went ok : all buttons enabled

            if (ConnectedTracker != null)
            {
                _loadCalibrationButton.Enabled = true;
                _viewCalibrationButton.Enabled = true;
            }
            else
            {
                _loadCalibrationButton.Enabled = false;
                _viewCalibrationButton.Enabled = false;
            }

            if (_homeForm.IsTracking)
            {
                _calibrateButton.Enabled = true;
                _trackStatus.Enabled = true;
                _calibrationTestButton.Enabled = true;
            }
            else
            {
                _calibrateButton.Enabled = false;
                _trackStatus.Enabled = false;
                _calibrationTestButton.Enabled = false;
                _gazeDotButton.Enabled = false;
            }

            _gazeDotButton.Text = GazeDot.Visible ? "Hide Gaze Dot" : "Show Gaze Dot";
            _saveCalibrationButton.Enabled = CalibrationIsOk;

            var itemsPerPage = _eventLog.Height / _eventLog.ItemHeight;
            _eventLog.TopIndex = _eventLog.Items.Count - itemsPerPage;
        }

        private void CalibrationForm_Load(object sender, EventArgs e)
        {
            if (ConnectedTracker == null)
            {
                _eventLog.Items.Add(DateTime.Now.Hour + ":" + DateTime.Now.Minute + ":" + DateTime.Now.Second + 
                    " - Could not connect to the eye tracker. Please make sure the eye ");
                _eventLog.Items.Add("tracker is connected to the computer and try to restart the application");
            }
            else
            {
                _eventLog.Items.Add(DateTime.Now.Hour + ":" + DateTime.Now.Minute + ":" + DateTime.Now.Second + " - Connected to: " + ConnectedTracker.GetUnitName());
                _eventLog.Items.Add("");
                _eventLog.Items.Add("Hi! Please position yourself in front of the eye tracker.");

                _eventLog.Items.Add("To ensure accurate eye gaze position you should calibrate the eye");
                _eventLog.Items.Add("tracker to your eyes. You can either start a new calibration or");
                _eventLog.Items.Add("load a calibration from file. When you're done you can go on and");
                _eventLog.Items.Add("play with the demo applications.");
                _eventLog.Items.Add("");
            }
            UpdateUiElements();
            UpdateDwellTimeLabel(DWELL_TIME_CALIBRATION_BUTTONS);
        }

        internal override void GazeDataReceived(object sender, GazeDataEventArgs e)
        {
            base.GazeDataReceived(sender, e);

            var gd = e.GazeDataItem;

            _trackStatus.OnGazeData(gd);

            if (GazeDataValid(gd))
            {
                if (IsUserGazingAt(_homeButton))
                    DoButtonInteraction(_homeButton, DWELL_TIME_CALIBRATION_BUTTONS);

                else
                {
                    FirstTimeUserLooksAtControl = true;
                    _homeButton.BackgroundImage = null;
                }
            }
        }

        #region ButtonClicks
        private void _calibrateButton_Click(object sender, EventArgs e)
        {
            CalibrationInstructions calibrationInstructions;

            if (_homeForm.FirstTimeUserSeesTheCalibrationInstructions)
            {
                calibrationInstructions = new CalibrationInstructions(10);
                _homeForm.FirstTimeUserSeesTheCalibrationInstructions = false;
            }
            else
                calibrationInstructions = new CalibrationInstructions(5);

            if (ConnectedTracker != null)
                ConnectedTracker.GazeDataReceived -= GazeDataReceived;

            var dialogResult = calibrationInstructions.ShowDialog();

            if (dialogResult != DialogResult.OK)
            {
                if (ConnectedTracker != null)
                    ConnectedTracker.GazeDataReceived += GazeDataReceived;
                return;
            }

            var runner = new CalibrationRunner();

            try
            {
                // Start a new calibration procedure
                var result = runner.RunCalibration(ConnectedTracker);

                // Show a calibration plot if everything went OK
                if (result != null)
                {
                    var resultForm = new CalibrationResultsForm(this);

                    if (ConnectedTracker != null)
                    {
                        ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                        ConnectedTracker.GazeDataReceived += resultForm.GazeDataReceived;
                    }
                    resultForm.SetPlotData(result);
                    
                    resultForm.Show();
                    Hide();

                    CalibrationIsOk = true;

                    // The user have now the options to save the calibration to file, or view it again.
                    // So we update the UI elements
                    UpdateUiElements();
                    _eventLog.Items.Add(DateTime.Now.Hour + ":" + DateTime.Now.Minute + ":" + DateTime.Now.Second +
                                        " - The calibration were successfully. If you are not happy with");
                    _eventLog.Items.Add("the result you can start a new calibration");
                    _eventLog.Items.Add("");
                    _eventLog.Items.Add("You can now start the other demo applications, who are available");
                    _eventLog.Items.Add("from the main menu screen. Try to stare at the main menu button for 2 seconds!");
                    _eventLog.Items.Add("");
                }
                else
                {
                    MessageBox.Show("Not enough data to create a good calibration (or calibration aborted). Please try again");
                }
            }
            catch (EyetrackerException ee)
            {
                MessageBox.Show("Failed to calibrate. Got exception " + ee,
                    "Calibration Failed",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
            if (ConnectedTracker != null) 
                ConnectedTracker.GazeDataReceived += GazeDataReceived;
        }

        private void _saveCalibrationButton_Click(object sender, EventArgs e)
        {
            ConnectedTracker.GazeDataReceived -= GazeDataReceived;
            var result = _saveFileDialog.ShowDialog();
            if (result == DialogResult.OK)
            {
                try
                {
                    var calibration = ConnectedTracker.GetCalibration();

                    using (var stream = _saveFileDialog.OpenFile())
                    using (var writer = new BinaryWriter(stream))
                    {
                        writer.Write(calibration.RawData);
                    }
                    _eventLog.Items.Add(DateTime.Now.Hour + ":" + DateTime.Now.Minute + ":" + DateTime.Now.Second +
                                        " - Successfully saved calibration data");
                }
                catch (EyetrackerException ee)
                {
                    MessageBox.Show("Failed to get calibration data. Got exception " + ee,
                        "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
                }
            }
            ConnectedTracker.GazeDataReceived += GazeDataReceived;
        }

        private void _viewCalibrationButton_Click(object sender, EventArgs e)
        {
            try
            {
                var calibration = ConnectedTracker.GetCalibration();
                var resultForm = new CalibrationResultsForm(this);
                
                if (ConnectedTracker != null)
                {
                    ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                    ConnectedTracker.GazeDataReceived += resultForm.GazeDataReceived;
                }
                resultForm.SetPlotData(calibration);
                
                resultForm.Show();
                Hide();

                _eventLog.Items.Add(DateTime.Now.Hour + ":" + DateTime.Now.Minute + ":" + DateTime.Now.Second +
                                    " - Viewing last calibration");
            }
            catch (EyetrackerException ee)
            {
                MessageBox.Show("Failed to get calibration data. Got exception " + ee,
                    "Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
            UpdateUiElements();
        }

        private void _loadCalibrationButton_Click(object sender, EventArgs e)
        {
            try
            {
                ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                var result = _openFileDialog.ShowDialog();
                if (result == DialogResult.OK)
                {
                    using (var stream = _openFileDialog.OpenFile())
                    using (var reader = new BinaryReader(stream))
                    {
                        byte[] data = reader.ReadBytes((int)stream.Length);
                        var calibration = new Calibration(data);
                        ConnectedTracker.SetCalibration(calibration);
                    }
                    _eventLog.Items.Add(DateTime.Now.Hour + ":" + DateTime.Now.Minute + ":" + DateTime.Now.Second + 
                        " - Successfully loaded calibration to eye tracker");

                    CalibrationIsOk = true;

                    // The user have now the options to save the calibration to file, or view it again.
                    // So we update the UI elements
                    UpdateUiElements();
                }
            }
            catch (EyetrackerException ee)
            {
                MessageBox.Show("Failed to load calibration data. Got exception " + ee,
                    "Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
            if (ConnectedTracker != null)
                ConnectedTracker.GazeDataReceived += GazeDataReceived;
        }

        private void _homeButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void _gazeDotButton_Click(object sender, EventArgs e)
        {
            if (!GazeDot.Visible)
            {
                GazeDot.Show();
                GazeDot.Size = GAZE_DOT_SIZE;
                GazeDot.Visible = true;
                _gazeDotButton.Text = "Hide Gaze Dot";
            }
            else
            {
                GazeDot.Visible = false;
                GazeDot.Hide();
                _gazeDotButton.Text = "Show Gaze Dot";
            }
        }
        #endregion

        private void CalibrationForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Clear the event log window for text
            _eventLog.Items.Clear();
            if (ConnectedTracker != null)
            {
                ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                ConnectedTracker.GazeDataReceived += _homeForm.GazeDataReceived;
            }
            _homeForm.Show();
        }

        private void _calibrationTestButton_Click(object sender, EventArgs e)
        {
             var calibrationTestForm = new CalibrationTestForm(this);
            if (_homeForm.IsTracking && ConnectedTracker != null)
            {
                ConnectedTracker.GazeDataReceived += calibrationTestForm.GazeDataReceived;
                ConnectedTracker.GazeDataReceived -= GazeDataReceived;
            }
            calibrationTestForm.Show();
            Hide();
        }
    }
}