﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Windows.Forms;
using Discovery.Calib;

namespace Discovery.Calibration_Test
{
    public partial class CalibrationTestForm : BaseForm
    {
        private double _gazeHitCircleDiameterAverage;
        private readonly List<double> _diameterList;
        private readonly CalibrationForm _calibrationForm;
        
        private const int MILLISECONDS_TO_CALCULATE_AVERAGE = 5000;
        private static TimeSpan DWELL_TIME_CALIBRATION_TEST_FORM = new TimeSpan(0, 0, 0, 1, 500);

        public CalibrationTestForm(CalibrationForm calibrationForm)
        {
            InitializeComponent();
            _calibrationForm = calibrationForm;
            _diameterList = new List<double>();
            SetTimerInterval(MILLISECONDS_TO_CALCULATE_AVERAGE);

            _calibrationTestStareArea.Paint += _calibrationTestStareArea_Paint;
           
            KeyPreview = true;
            KeyDown += CalibrationTestForm_KeyDown;
        }

        // Draws the dot at the middle of the testStareArea
        private void _calibrationTestStareArea_Paint(object sender, PaintEventArgs e)
        {
            var brush = new SolidBrush(Color.Red);
            e.Graphics.FillEllipse(brush, new Rectangle((_calibrationTestStareArea.Width / 2)-4, (_calibrationTestStareArea.Height / 2)-4, 8, 8));
        }

        private void CalibrationTestForm_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control && e.KeyCode == Keys.Add)
            {
                Log.Write("The Calibration Test Application: Dwell time: " + DWELL_TIME_CALIBRATION_TEST_FORM.TotalMilliseconds + " ms");
                DWELL_TIME_CALIBRATION_TEST_FORM = IncreaseDwellTime(DWELL_TIME_CALIBRATION_TEST_FORM);
            }
            if (e.Control && e.KeyCode == Keys.Subtract)
            {
                Log.Write("The Calibration Test Application: Dwell time: " + DWELL_TIME_CALIBRATION_TEST_FORM.TotalMilliseconds + " ms");
                DWELL_TIME_CALIBRATION_TEST_FORM = DecreaseDwellTime(DWELL_TIME_CALIBRATION_TEST_FORM);
            }

            UpdateDwellTimeLabel(DWELL_TIME_CALIBRATION_TEST_FORM);
        }

        internal override void GazeDataReceived(object sender, Tobii.Eyetracking.Sdk.GazeDataEventArgs e)
        {
            base.GazeDataReceived(sender, e);

            var gd = e.GazeDataItem;

            if (GazeDataValid(gd))
            {
                if (IsUserGazingAt(_calibrationTestStareArea))
                {
                    StartTimer(null);
                    if (UserHasLookedLongEnough(DWELL_TIME_CALIBRATION_TEST_FORM))
                    {
                        var gazeHitCircleDiameter = ComputeGazeHitCircleDiameter();
                        _diameterList.Add(gazeHitCircleDiameter);

                        _gazeHitCircleDiameterAverage = _diameterList.Sum() / _diameterList.Count;

                        _calibrationTestListBox.Items.Add("Gaze diameter: " + gazeHitCircleDiameter);

                        // Scrolling the listBox down
                        var itemsPerPage = _calibrationTestListBox.Height / _calibrationTestListBox.ItemHeight;
                        _calibrationTestListBox.TopIndex = _calibrationTestListBox.Items.Count - itemsPerPage;
                    }
                }
                else if (IsUserGazingAt(_homeButton))
                    DoButtonInteraction(_homeButton);
                else
                {
                    StopTimer(null);
                    FirstTimeUserLooksAtControl = true;
                    _homeButton.BackgroundImage = null;
                }
            }
        }

        private double ComputeGazeHitCircleDiameter()
        {
            var minimumX = Screen.PrimaryScreen.Bounds.Width / 2;
            var maximumX = Screen.PrimaryScreen.Bounds.Width / 2;

            var minimumY = Screen.PrimaryScreen.Bounds.Height / 2;
            var maximumY = Screen.PrimaryScreen.Bounds.Height / 2;

            foreach (var x in _gazeDataXLowPassedHistory.Queue)
            {
                if (x < minimumX)
                    minimumX = x;
                if (x > maximumX)
                    maximumX = x;
            }
            foreach (var y in _gazeDataYLowPassedHistory.Queue)
            {
                if (y < minimumY)
                    minimumY = y;
                if (y > maximumY)
                    maximumY = y;
            }
            return maximumX - minimumX > maximumY - minimumY ? maximumX - minimumX : maximumY - minimumY;
        }

        internal override void HandleTimerTick(object sender, EventArgs e)
        {
            if (_colorIndex >= _colors.Count)
                _colorIndex = 0;
            var myPen = new Pen(_colors[++_colorIndex]);

            var formGraphics = _calibrationTestStareArea.CreateGraphics();
            formGraphics.DrawEllipse(myPen, new Rectangle((_calibrationTestStareArea.Width / 2) - (int)_gazeHitCircleDiameterAverage / 2, _calibrationTestStareArea.Height / 2 - (int)_gazeHitCircleDiameterAverage / 2, (int)_gazeHitCircleDiameterAverage, (int)_gazeHitCircleDiameterAverage));
            myPen.Dispose();
            formGraphics.Dispose();
            _averageList.Items.Add("Average diameter = " + _gazeHitCircleDiameterAverage);
            
            _diameterList.Clear();

            var itemsPerPage = _calibrationTestListBox.Height / _calibrationTestListBox.ItemHeight;
            _calibrationTestListBox.TopIndex = _calibrationTestListBox.Items.Count - itemsPerPage;
        }
        private void _homeButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void CalibrationTestForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (_calibrationForm.ConnectedTracker != null)
            {
                _calibrationForm.ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                _calibrationForm.ConnectedTracker.GazeDataReceived += _calibrationForm.GazeDataReceived;
            }
                
            _calibrationForm.Show();
        }

        private void CalibrationTestForm_Load(object sender, EventArgs e)
        {
            UpdateDwellTimeLabel(DWELL_TIME_CALIBRATION_TEST_FORM);
        }
    }
}
