﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Discovery.Properties;
using Tobii.Eyetracking.Sdk;

namespace Discovery.Controls
{
    /*
     * The Controls Application has some commonly used Windows Controls that users can control with
     * their eyes. The main function is GazeDataReceived that overrides the base class' implementation
     * and checks whether the user stares at one of the controls or not. If the user stares at a control until the
     * dwell time has passed, an action is taken on the control.
     * 
     * Haakon Raudsandmoen and Boerge Roedsjoe, Trondheim, 16. February 2012
     */
    public partial class ControlsButtonsForm : BaseForm
    {
        private readonly HomeForm _homeForm;
        private readonly List<Button> _buttons;
        private Button _lastLookedAtButton;

        private static TimeSpan DWELL_TIME_CONTROLS_BUTTONS = new TimeSpan(0, 0, 0, 1);
        private static TimeSpan DWELL_TIME_CONTROLS_BUTTONS_FORM = new TimeSpan(0, 0, 0, 1, 500);

        private int _color1index;
        private int _color2index;
        private int _color3index;
        private int _color4index;
        private int _color5index;
        private int _color6index;

        internal IEyetracker ConnectedTracker { get; set; }
        
        // Constructor that initalizes the layout
        public ControlsButtonsForm(HomeForm homeForm)
        {
            InitializeComponent();
            _homeForm = homeForm;
            ConnectedTracker = _homeForm.ConnectedTracker;
            SetTimerInterval(500);

            _lastLookedAtButton = new Button();
            _buttons = new List<Button>
                          {_homeButton,
                              _controlsNextButton,
                              _controlsPreviousButton,
                           _colorButton2, 
                           _colorButton3, 
                           _colorButton4, 
                           _colorButton5, 
                           _colorButton6, 
                           _colorButton1};

            KeyPreview = true;
            KeyDown += ControlsForm_KeyDown;
        }

        private void ControlsForm_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control && e.KeyCode == Keys.Add)
            {
                Log.Write("Buttons Application: Dwell time: " + DWELL_TIME_CONTROLS_BUTTONS_FORM.TotalMilliseconds + " ms");
                DWELL_TIME_CONTROLS_BUTTONS_FORM = IncreaseDwellTime(DWELL_TIME_CONTROLS_BUTTONS_FORM);
            }
            if (e.Control && e.KeyCode == Keys.Subtract)
            {
                Log.Write("Buttons Application: Dwell time: " + DWELL_TIME_CONTROLS_BUTTONS_FORM.TotalMilliseconds + " ms");
                DWELL_TIME_CONTROLS_BUTTONS_FORM = DecreaseDwellTime(DWELL_TIME_CONTROLS_BUTTONS_FORM);
            }

            UpdateDwellTimeLabel(DWELL_TIME_CONTROLS_BUTTONS_FORM);
        }

        // This method override's the base class' implementation, and uses the base class' functions
        // to check if the user is staring at any controls in The Controls Application. If the user
        // stares at a button, a call to DoButtonInteraction(control) is done.
        internal override void GazeDataReceived(object sender, GazeDataEventArgs e)
        {
            base.GazeDataReceived(sender, e);

            var gd = e.GazeDataItem;

            if (GazeDataValid(gd))
            {
                if (IsUserGazingAt(_homeButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _homeButton);
                    DoButtonInteraction(_homeButton, DWELL_TIME_CONTROLS_BUTTONS_FORM);
                }
                else if (IsUserGazingAt(_controlsNextButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _controlsNextButton);
                    DoButtonInteraction(_controlsNextButton, DWELL_TIME_CONTROLS_BUTTONS_FORM);
                }
                else if (IsUserGazingAt(_controlsPreviousButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _controlsPreviousButton);
                    DoButtonInteraction(_controlsPreviousButton, DWELL_TIME_CONTROLS_BUTTONS);
                }
                else if (IsUserGazingAt(_colorButton2))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _colorButton2);
                    DoButtonInteractionForColorButtons(_colorButton2, DWELL_TIME_CONTROLS_BUTTONS);

                    if (_lastLookedAtButton != _colorButton2)
                        _colorButton2.BackgroundImage = Resources.eye_hover;
                    _lastLookedAtButton = _colorButton2;
                }

                else if (IsUserGazingAt(_colorButton3))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _colorButton3);
                    DoButtonInteractionForColorButtons(_colorButton3, DWELL_TIME_CONTROLS_BUTTONS);
                    
                    if (_lastLookedAtButton != _colorButton3)
                        _colorButton3.BackgroundImage = Resources.eye_hover;    
                    _lastLookedAtButton = _colorButton3;
                }

                else if (IsUserGazingAt(_colorButton4))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _colorButton4);
                    DoButtonInteractionForColorButtons(_colorButton4, DWELL_TIME_CONTROLS_BUTTONS);

                    if (_lastLookedAtButton != _colorButton4)
                        _colorButton4.BackgroundImage = Resources.eye_hover;
                    _lastLookedAtButton = _colorButton4;
                }

                else if (IsUserGazingAt(_colorButton5))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _colorButton5);
                    DoButtonInteractionForColorButtons(_colorButton5, DWELL_TIME_CONTROLS_BUTTONS);

                    if (_lastLookedAtButton != _colorButton5)
                        _colorButton5.BackgroundImage = Resources.eye_hover;
                    _lastLookedAtButton = _colorButton5;
                }
                
                else if (IsUserGazingAt(_colorButton6))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _colorButton6);
                    DoButtonInteractionForColorButtons(_colorButton6, DWELL_TIME_CONTROLS_BUTTONS);

                    if (_lastLookedAtButton != _colorButton6)
                        _colorButton6.BackgroundImage = Resources.eye_hover;
                    _lastLookedAtButton = _colorButton6;
                }
                
                else if (IsUserGazingAt(_colorButton1))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _colorButton1);
                    DoButtonInteractionForColorButtons(_colorButton1, DWELL_TIME_CONTROLS_BUTTONS_FORM);

                    if (_lastLookedAtButton != _colorButton1)
                        _colorButton1.BackgroundImage = Resources.eye_hover;
                    _lastLookedAtButton = _colorButton1;
                }

                else
                {
                    FirstTimeUserLooksAtControl = true;
                    
                    RemoveHoverEffectFromButtons(_buttons);
                    _lastLookedAtButton = null;
                }
            }
        }


        // Close the application if home-button is clicked
        private void _homeButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        // Determines the action of what to do when one of the colourbuttons are clicked
        private void _colorButton_Click(object sender, EventArgs e)
        {
            var button = (Button) sender;
            if (button == _colorButton2)
            {
                if (_color1index >= _minicolors.Count)
                    _color1index = 0;
                _colorButton2.BackColor = _minicolors[_color1index++];
            }
            if (button == _colorButton3)
            {
                if (_color2index >= _minicolors.Count)
                    _color2index = 0;
                _colorButton3.BackColor = _minicolors[_color2index++];
            }
            if (button == _colorButton4)
            {
                if (_color3index >= _minicolors.Count)
                    _color3index = 0;
                _colorButton4.BackColor = _minicolors[_color3index++];
            }
            if (button == _colorButton5)
            {
                if (_color4index >= _minicolors.Count)
                    _color4index = 0;
                _colorButton5.BackColor = _minicolors[_color4index++];
            }
            if (button == _colorButton6)
            {
                if (_color5index >= _minicolors.Count)
                    _color5index = 0;
                _colorButton6.BackColor = _minicolors[_color5index++];
            }
            if (button == _colorButton1)
            {
                if (_color6index >= _minicolors.Count)
                    _color6index = 0;
                _colorButton1.BackColor = _minicolors[_color6index++];
            }
        }

        // Shuts down this form, stops recieving gaze data, shows the main menu and starts
        // recieving gaze data for the main menu
        private void ControlsForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (_homeForm.ConnectedTracker != null)
            {
                _homeForm.ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                _homeForm.ConnectedTracker.GazeDataReceived += _homeForm.GazeDataReceived;
            }
            _homeForm.Show();
        }

        private void ControlsForm_Load(object sender, EventArgs e)
        {
            UpdateDwellTimeLabel(DWELL_TIME_CONTROLS_BUTTONS_FORM);
        }

        private void _controlsNextButton_Click(object sender, EventArgs e)
        {
            var controlsLinksForm = new ControlsLinksForm(_homeForm);
            if (_homeForm.IsTracking && ConnectedTracker != null)
            {
                ConnectedTracker.GazeDataReceived += controlsLinksForm.GazeDataReceived;
                ConnectedTracker.GazeDataReceived -= GazeDataReceived;
            }
            controlsLinksForm.Show();
            Hide();
        }

        private void _controlsPreviousButton_Click(object sender, EventArgs e)
        {
            var controlsTrackbarsForm = new ControlsTrackbarsForm(_homeForm);
            if (_homeForm.IsTracking && ConnectedTracker != null)
            {
                ConnectedTracker.GazeDataReceived += controlsTrackbarsForm.GazeDataReceived;
                ConnectedTracker.GazeDataReceived -= GazeDataReceived;
            }
            controlsTrackbarsForm.Show();
            Hide();
        }

    }
}
