﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using Tobii.Eyetracking.Sdk;

namespace Discovery.Controls
{
    /*
     * The Controls Application has some commonly used Windows Controls that users can control with
     * their eyes. The main function is GazeDataReceived that overrides the base class' implementation
     * and checks whether the user stares at one of the controls or not. If the user stares at a control until the
     * dwell time has passed, an action is taken on the control.
     * 
     * Haakon Raudsandmoen and Boerge Roedsjoe, Trondheim, 16. February 2012
     */
    public partial class ControlsCheckboxesForm : BaseForm
    {
        private readonly HomeForm _homeForm;
        private readonly List<Button> _buttons;

        internal List<Control> _checkboxes;

        private static TimeSpan DWELL_TIME_CONTROLS_CHECKBOXES_BUTTONS = new TimeSpan(0, 0, 0, 1, 500);
        private static TimeSpan DWELL_TIME_CONTROLS_CHECKBOXES = new TimeSpan(0, 0, 0, 0, 500);

        internal IEyetracker ConnectedTracker { get; set; }

        // Constructor that initalizes the layout
        public ControlsCheckboxesForm(HomeForm homeForm)
        {
            InitializeComponent();
            _homeForm = homeForm;
            ConnectedTracker = _homeForm.ConnectedTracker;
            SetTimerInterval((int)DWELL_TIME_CONTROLS_CHECKBOXES.TotalMilliseconds);

            _buttons = new List<Button> { _homeButton, _controlsNextButton, _controlsPreviousButton };

            _checkboxes = new List<Control> { 
                _checkBox1, 
                _checkBox2,
                _checkBox3, 
                _checkBox4, 
                _checkBox5, 
                _checkBox6, 
                _checkBox7, 
                _checkBox8,
                _checkBox9, 
                _checkBox10, 
                _checkBox11, 
                _checkBox12,
                _checkBox13, 
                _checkBox14, 
                _checkBox15, 
                _checkBox16};

            KeyPreview = true;
            KeyDown += ControlsForm_KeyDown;
        }

        private void ControlsForm_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control && e.KeyCode == Keys.Add)
            {
                DWELL_TIME_CONTROLS_CHECKBOXES = IncreaseDwellTime(DWELL_TIME_CONTROLS_CHECKBOXES);
                Log.Write("Checkboxes Application: Dwell time checkboxes: " + DWELL_TIME_CONTROLS_CHECKBOXES.TotalMilliseconds + " ms");
            }
            if (e.Control && e.KeyCode == Keys.Subtract)
            {
                DWELL_TIME_CONTROLS_CHECKBOXES = DecreaseDwellTime(DWELL_TIME_CONTROLS_CHECKBOXES);
                Log.Write("Checkboxes Application: Dwell time checkboxes: " + DWELL_TIME_CONTROLS_CHECKBOXES.TotalMilliseconds + " ms");
            }

            UpdateDwellTimeLabel(DWELL_TIME_CONTROLS_CHECKBOXES);
        }

        // This method override's the base class' implementation, and uses the base class' functions
        // to check if the user is staring at any controls in The Controls Application. If the user
        // stares at a button, a call to DoButtonInteraction(control) is done.
        internal override void GazeDataReceived(object sender, GazeDataEventArgs e)
        {
            base.GazeDataReceived(sender, e);

            var gd = e.GazeDataItem;

            if (GazeDataValid(gd))
            {
                if (IsUserGazingAt(_homeButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _homeButton);
                    DoButtonInteraction(_homeButton, DWELL_TIME_CONTROLS_CHECKBOXES_BUTTONS);
                }
                else if (IsUserGazingAt(_checkBox1))
                {
                    _checkBox1.Font = AddBoldHover(_checkBox1.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox1.Checked = !_checkBox1.Checked;
                }
                else if (IsUserGazingAt(_checkBox2))
                {
                    _checkBox2.Font = AddBoldHover(_checkBox2.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox2.Checked = !_checkBox2.Checked;
                }
                else if (IsUserGazingAt(_checkBox3))
                {
                    _checkBox3.Font = AddBoldHover(_checkBox3.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox3.Checked = !_checkBox3.Checked;
                }
                else if (IsUserGazingAt(_checkBox4))
                {
                    _checkBox4.Font = AddBoldHover(_checkBox4.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox4.Checked = !_checkBox4.Checked;
                }
                else if (IsUserGazingAt(_checkBox5))
                {
                    _checkBox5.Font = AddBoldHover(_checkBox5.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox5.Checked = !_checkBox5.Checked;
                }
                else if (IsUserGazingAt(_checkBox6))
                {
                    _checkBox6.Font = AddBoldHover(_checkBox6.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox6.Checked = !_checkBox6.Checked;
                }
                else if (IsUserGazingAt(_checkBox7))
                {
                    _checkBox7.Font = AddBoldHover(_checkBox7.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox7.Checked = !_checkBox7.Checked;
                }
                else if (IsUserGazingAt(_checkBox8))
                {
                    _checkBox8.Font = AddBoldHover(_checkBox8.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox8.Checked = !_checkBox8.Checked;
                }
                else if (IsUserGazingAt(_checkBox9))
                {
                    _checkBox9.Font = AddBoldHover(_checkBox9.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox9.Checked = !_checkBox9.Checked;
                }

                else if (IsUserGazingAt(_checkBox10))
                {
                    _checkBox10.Font = AddBoldHover(_checkBox10.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox10.Checked = !_checkBox10.Checked;
                }

                else if (IsUserGazingAt(_checkBox11))
                {
                    _checkBox11.Font = AddBoldHover(_checkBox11.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox11.Checked = !_checkBox11.Checked;
                }

                else if (IsUserGazingAt(_checkBox12))
                {
                    _checkBox12.Font = AddBoldHover(_checkBox12.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox12.Checked = !_checkBox12.Checked;
                }
                else if (IsUserGazingAt(_checkBox13))
                {
                    _checkBox13.Font = AddBoldHover(_checkBox13.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox13.Checked = !_checkBox13.Checked;
                }
                else if (IsUserGazingAt(_checkBox14))
                {
                    _checkBox14.Font = AddBoldHover(_checkBox14.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox14.Checked = !_checkBox14.Checked;

                }
                else if (IsUserGazingAt(_checkBox15))
                {
                    _checkBox15.Font = AddBoldHover(_checkBox15.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox15.Checked = !_checkBox15.Checked;
                }
                else if (IsUserGazingAt(_checkBox16))
                {
                    _checkBox16.Font = AddBoldHover(_checkBox16.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox16.Checked = !_checkBox16.Checked;
                }
                else if (IsUserGazingAt(_controlsNextButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _controlsNextButton);
                    DoButtonInteraction(_controlsNextButton, DWELL_TIME_CONTROLS_CHECKBOXES_BUTTONS);
                }
                else if (IsUserGazingAt(_controlsPreviousButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _controlsPreviousButton);
                    DoButtonInteraction(_controlsPreviousButton, DWELL_TIME_CONTROLS_CHECKBOXES_BUTTONS);
                }
                else
                {
                    FirstTimeUserLooksAtControl = true;
                    RemoveHoverEffectFromButtons(_buttons);
                    RemoveBoldHover(_checkBox1.Font, _checkboxes);
                }
            }
        }

        // Close the application if home-button is clicked
        private void _homeButton_Click(object sender, EventArgs e)
        {
            Close();
        }
        // Method for bold hover effect
        static internal Font AddBoldHover(Font font, List<Control> controls)
        {
            RemoveBoldHover(font, controls);
            font = new Font(font.Name, font.Size,
            FontStyle.Bold, font.Unit,
            font.GdiCharSet, font.GdiVerticalFont);

            return font;
        }
        static internal void RemoveBoldHover(Font font, List<Control> controls)
        {
            foreach (var c in controls)
            {
                c.Font = new Font(font.Name, font.Size,
                        FontStyle.Regular, font.Unit,
                        font.GdiCharSet, font.GdiVerticalFont);
            }
        }

        // Shuts down this form, stops recieving gaze data, shows the main menu and starts
        // recieving gaze data for the main menu
        private void ControlsCheckboxesForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (_homeForm.ConnectedTracker != null)
            {
                _homeForm.ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                _homeForm.ConnectedTracker.GazeDataReceived += _homeForm.GazeDataReceived;
            }
            _homeForm.Show();
        }

        private void ControlsCheckboxesForm_Load(object sender, EventArgs e)
        {
            UpdateDwellTimeLabel(DWELL_TIME_CONTROLS_CHECKBOXES);
        }
        private void _controlsNextButton_Click(object sender, EventArgs e)
        {
            var controlsTrackbarsForm = new ControlsTrackbarsForm(_homeForm);
            if (_homeForm.IsTracking && ConnectedTracker != null)
            {
                ConnectedTracker.GazeDataReceived += controlsTrackbarsForm.GazeDataReceived;
                ConnectedTracker.GazeDataReceived -= GazeDataReceived;
            }
            controlsTrackbarsForm.Show();
            Hide();
        }

        private void _controlsPreviousButton_Click(object sender, EventArgs e)
        {
            var controlsLinksForm = new ControlsLinksForm(_homeForm);
            if (_homeForm.IsTracking && ConnectedTracker != null)
            {
                ConnectedTracker.GazeDataReceived += controlsLinksForm.GazeDataReceived;
                ConnectedTracker.GazeDataReceived -= GazeDataReceived;
            }
            controlsLinksForm.Show();
            Hide();
        }
    }
}
