﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Tobii.Eyetracking.Sdk;

namespace Discovery.Controls
{
    /*
     * The Controls Application has some commonly used Windows Controls that users can control with
     * their eyes. The main function is GazeDataReceived that overrides the base class' implementation
     * and checks whether the user stares at one of the controls or not. If the user stares at a control until the
     * dwell time has passed, an action is taken on the control.
     * 
     * Haakon Raudsandmoen and Boerge Roedsjoe, Trondheim, 16. February 2012
     */
    public partial class ControlsLinksForm : BaseForm
    {
        private readonly HomeForm _homeForm;
        private readonly List<Button> _buttons;

        private static TimeSpan DWELL_TIME_CONTROLS_LINKS_BUTTONS = new TimeSpan(0, 0, 0, 1, 500);
        private static TimeSpan DWELL_TIME_CONTROLS_LINKS = new TimeSpan(0, 0, 0, 1);

        internal IEyetracker ConnectedTracker { get; set; }
        
        // Constructor that initalizes the layout
        public ControlsLinksForm(HomeForm homeForm)
        {
            InitializeComponent();
            _homeForm = homeForm;
            ConnectedTracker = _homeForm.ConnectedTracker;
            SetTimerInterval(500);

            _buttons = new List<Button> {_homeButton, _controlsNextButton, _controlsPreviousButton};

            KeyPreview = true;
            KeyDown += ControlsForm_KeyDown;
        }

        private void ControlsForm_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control && e.KeyCode == Keys.Add)
            {
                Log.Write("Links Application: Dwell time: " + DWELL_TIME_CONTROLS_LINKS.TotalMilliseconds + " ms");
                DWELL_TIME_CONTROLS_LINKS = IncreaseDwellTime(DWELL_TIME_CONTROLS_LINKS);
            }
            if (e.Control && e.KeyCode == Keys.Subtract)
            {
                Log.Write("Links Application: Dwell time: " + DWELL_TIME_CONTROLS_LINKS.TotalMilliseconds + " ms");
                DWELL_TIME_CONTROLS_LINKS = DecreaseDwellTime(DWELL_TIME_CONTROLS_LINKS);
            }

            UpdateDwellTimeLabel(DWELL_TIME_CONTROLS_LINKS);
        }

        // This method override's the base class' implementation, and uses the base class' functions
        // to check if the user is staring at any controls in The Controls Application. If the user
        // stares at a button, a call to DoButtonInteraction(control) is done.
        internal override void GazeDataReceived(object sender, GazeDataEventArgs e)
        {
            base.GazeDataReceived(sender, e);

            var gd = e.GazeDataItem;

            if (GazeDataValid(gd))
            {
                if (IsUserGazingAt(_homeButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _homeButton);
                    DoButtonInteraction(_homeButton, DWELL_TIME_CONTROLS_LINKS_BUTTONS);
                }
                else if (IsUserGazingAt(_controlsNextButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _controlsNextButton);
                    DoButtonInteraction(_controlsNextButton, DWELL_TIME_CONTROLS_LINKS_BUTTONS);
                }
                else if (IsUserGazingAt(_controlsPreviousButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _controlsPreviousButton);
                    DoButtonInteraction(_controlsPreviousButton, DWELL_TIME_CONTROLS_LINKS_BUTTONS);
                }
                else if (IsUserGazingAt(_linkLabel1))
                {
                    _linkLabel1.LinkBehavior = LinkBehavior.AlwaysUnderline;
                    toolTip1.Show("http://www.yr.no", _linkLabel1);

                    if (UserHasLookedLongEnough())
                    {
                        InvokeOnClick(_linkLabel1, new MouseEventArgs(MouseButtons.Left, 0, 0, 0, 0));
                        _linkLabel1.LinkBehavior = LinkBehavior.NeverUnderline;
                        toolTip1.Hide(_linkLabel1);
                    }
                }

                else if (IsUserGazingAt(_linkLabel2))
                {
                    _linkLabel2.LinkBehavior = LinkBehavior.AlwaysUnderline;
                    toolTip1.Show("http://www.ntnu.no", _linkLabel2);
                    if (UserHasLookedLongEnough())
                    {
                        InvokeOnClick(_linkLabel2, new MouseEventArgs(MouseButtons.Left, 0, 0, 0, 0));
                        _linkLabel2.LinkBehavior = LinkBehavior.NeverUnderline;
                        toolTip1.Hide(_linkLabel2);
                    }
                }

                else if (IsUserGazingAt(_linkLabel3))
                {
                    _linkLabel3.LinkBehavior = LinkBehavior.AlwaysUnderline;
                    toolTip1.Show("http://www.google.no", _linkLabel3);
                    if (UserHasLookedLongEnough())
                    {
                        InvokeOnClick(_linkLabel3, new MouseEventArgs(MouseButtons.Left, 0, 0, 0, 0));
                        _linkLabel3.LinkBehavior = LinkBehavior.NeverUnderline;
                        toolTip1.Hide(_linkLabel3);
                    }
                }

                else if (IsUserGazingAt(_linkLabel4))
                {
                    _linkLabel4.LinkBehavior = LinkBehavior.AlwaysUnderline;
                    toolTip1.Show("http://www.storm.no", _linkLabel4);
                    if (UserHasLookedLongEnough())
                    {
                        InvokeOnClick(_linkLabel4, new MouseEventArgs(MouseButtons.Left, 0, 0, 0, 0));
                        _linkLabel4.LinkBehavior = LinkBehavior.NeverUnderline;
                        toolTip1.Hide(_linkLabel4);
                    }
                }
                else if (IsUserGazingAt(_linkLabel1NoToolTip))
                {
                    _linkLabel1NoToolTip.LinkBehavior = LinkBehavior.AlwaysUnderline;
                    if (UserHasLookedLongEnough())
                    {
                        InvokeOnClick(_linkLabel1NoToolTip, new MouseEventArgs(MouseButtons.Left, 0, 0, 0, 0));
                        _linkLabel1NoToolTip.LinkBehavior = LinkBehavior.NeverUnderline;
                    }
                }

                else if (IsUserGazingAt(_linkLabel2NoToolTip))
                {
                    _linkLabel2NoToolTip.LinkBehavior = LinkBehavior.AlwaysUnderline;
                    if (UserHasLookedLongEnough())
                    {
                        InvokeOnClick(_linkLabel2NoToolTip, new MouseEventArgs(MouseButtons.Left, 0, 0, 0, 0));
                        _linkLabel2NoToolTip.LinkBehavior = LinkBehavior.NeverUnderline;
                    }
                }

                else if (IsUserGazingAt(_linkLabel3NoToolTip))
                {
                    _linkLabel3NoToolTip.LinkBehavior = LinkBehavior.AlwaysUnderline;
                    if (UserHasLookedLongEnough())
                    {
                        InvokeOnClick(_linkLabel3NoToolTip, new MouseEventArgs(MouseButtons.Left, 0, 0, 0, 0));
                        _linkLabel3NoToolTip.LinkBehavior = LinkBehavior.NeverUnderline;
                    }
                }

                else if (IsUserGazingAt(_linkLabel4NoToolTip))
                {
                    _linkLabel4NoToolTip.LinkBehavior = LinkBehavior.AlwaysUnderline;
                    if (UserHasLookedLongEnough())
                    {
                        InvokeOnClick(_linkLabel4NoToolTip, new MouseEventArgs(MouseButtons.Left, 0, 0, 0, 0));
                        _linkLabel4NoToolTip.LinkBehavior = LinkBehavior.NeverUnderline;
                    }
                }

                else
                {
                    FirstTimeUserLooksAtControl = true;
                    RemoveHoverEffectFromButtons(_buttons);

                    toolTip1.Hide(_linkLabel1);
                    toolTip1.Hide(_linkLabel2);
                    toolTip1.Hide(_linkLabel3);
                    toolTip1.Hide(_linkLabel4);

                    _linkLabel1.LinkBehavior = LinkBehavior.NeverUnderline;
                    _linkLabel2.LinkBehavior = LinkBehavior.NeverUnderline;
                    _linkLabel3.LinkBehavior = LinkBehavior.NeverUnderline;
                    _linkLabel4.LinkBehavior = LinkBehavior.NeverUnderline;

                    _linkLabel1NoToolTip.LinkBehavior = LinkBehavior.NeverUnderline;
                    _linkLabel2NoToolTip.LinkBehavior = LinkBehavior.NeverUnderline;
                    _linkLabel3NoToolTip.LinkBehavior = LinkBehavior.NeverUnderline;
                    _linkLabel4NoToolTip.LinkBehavior = LinkBehavior.NeverUnderline;
                }
            }
        }

        // Close the application if home-button is clicked
        private void _homeButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        // If a link is clicked, we open a popup with a congratulations text
        private void _linkLabel_Click(object sender, EventArgs e)
        {
            var linkLabel = (LinkLabel)sender;

            if (linkLabel == _linkLabel1)
            {
                var popUpForm = new PopUpForm(this, "yr.no",
                              "Congratulations! \n You succsessfully clicked the link. \n \n Click OK to continue with the rest of the tasks.");
                if (_homeForm.ConnectedTracker != null)
                {
                    _homeForm.ConnectedTracker.GazeDataReceived += popUpForm.GazeDataReceived;
                    _homeForm.ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                }
                popUpForm.Show();
            }
            if (linkLabel == _linkLabel2)
            {
                var popUpForm = new PopUpForm(this, "ntnu.no",
                              "Congratulations! \n You succsessfully clicked the link. \n \n Click OK to continue with the rest of the tasks.");
                if (_homeForm.ConnectedTracker != null)
                {
                    _homeForm.ConnectedTracker.GazeDataReceived += popUpForm.GazeDataReceived;
                    _homeForm.ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                }
                popUpForm.Show();
            }
            if (linkLabel == _linkLabel3)
            {
                var popUpForm = new PopUpForm(this, "google.no",
                              "Congratulations! \n You succsessfully clicked the link. \n \n Click OK to continue with the rest of the tasks.");
                if (_homeForm.ConnectedTracker != null)
                {
                    _homeForm.ConnectedTracker.GazeDataReceived += popUpForm.GazeDataReceived;
                    _homeForm.ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                }
                popUpForm.Show();
            }
            if (linkLabel == _linkLabel4)
            {
                var popUpForm = new PopUpForm(this, "storm.no",
                              "Congratulations! \n You succsessfully clicked the link. \n \n Click OK to continue with the rest of the tasks.");
                if (_homeForm.ConnectedTracker != null)
                {
                    _homeForm.ConnectedTracker.GazeDataReceived += popUpForm.GazeDataReceived;
                    _homeForm.ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                }
                popUpForm.Show();
            }
            if (linkLabel == _linkLabel1NoToolTip)
            {
                var popUpForm = new PopUpForm(this, "yr.no",
                              "Congratulations! \n You succsessfully clicked the link. \n \n Click OK to continue with the rest of the tasks.");
                if (_homeForm.ConnectedTracker != null)
                {
                    _homeForm.ConnectedTracker.GazeDataReceived += popUpForm.GazeDataReceived;
                    _homeForm.ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                }
                popUpForm.Show();
            }
            if (linkLabel == _linkLabel2NoToolTip)
            {
                var popUpForm = new PopUpForm(this, "ntnu.no",
                              "Congratulations! \n You succsessfully clicked the link. \n \n Click OK to continue with the rest of the tasks.");
                if (_homeForm.ConnectedTracker != null)
                {
                    _homeForm.ConnectedTracker.GazeDataReceived += popUpForm.GazeDataReceived;
                    _homeForm.ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                }
                popUpForm.Show();
            }
            if (linkLabel == _linkLabel3NoToolTip)
            {
                var popUpForm = new PopUpForm(this, "google.no",
                              "Congratulations! \n You succsessfully clicked the link. \n \n Click OK to continue with the rest of the tasks.");
                if (_homeForm.ConnectedTracker != null)
                {
                    _homeForm.ConnectedTracker.GazeDataReceived += popUpForm.GazeDataReceived;
                    _homeForm.ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                }
                popUpForm.Show();
            }
            if (linkLabel == _linkLabel4NoToolTip)
            {
                var popUpForm = new PopUpForm(this, "storm.no",
                              "Congratulations! \n You succsessfully clicked the link. \n \n Click OK to continue with the rest of the tasks.");
                if (_homeForm.ConnectedTracker != null)
                {
                    _homeForm.ConnectedTracker.GazeDataReceived += popUpForm.GazeDataReceived;
                    _homeForm.ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                }
                popUpForm.Show();
            }
        }
        

        // Shuts down this form, stops recieving gaze data, shows the main menu and starts
        // recieving gaze data for the main menu
        private void ControlsLinksForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (_homeForm.ConnectedTracker != null)
            {
                _homeForm.ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                _homeForm.ConnectedTracker.GazeDataReceived += _homeForm.GazeDataReceived;
            }
            _homeForm.Show();
        }

        private void ControlsLinksForm_Load(object sender, EventArgs e)
        {
            UpdateDwellTimeLabel(DWELL_TIME_CONTROLS_LINKS);
        }
        private void _controlsNextButton_Click(object sender, EventArgs e)
        {
            var controlsCheckboxesForm = new ControlsCheckboxesForm(_homeForm);
            if (_homeForm.IsTracking && ConnectedTracker != null)
            {
                ConnectedTracker.GazeDataReceived += controlsCheckboxesForm.GazeDataReceived;
                ConnectedTracker.GazeDataReceived -= GazeDataReceived;
            }
            controlsCheckboxesForm.Show();
            Hide();
        }

        private void _controlsPreviousButton_Click(object sender, EventArgs e)
        {
            var controlsButtonsForm = new ControlsButtonsForm(_homeForm);
            if (_homeForm.IsTracking && ConnectedTracker != null)
            {
                ConnectedTracker.GazeDataReceived += controlsButtonsForm.GazeDataReceived;
                ConnectedTracker.GazeDataReceived -= GazeDataReceived;
            }
            controlsButtonsForm.Show();
            Hide();
        }
    }
}
