﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Tobii.Eyetracking.Sdk;

namespace Discovery.Controls
{
    /*
     * The Controls Application has some commonly used Windows Controls that users can control with
     * their eyes. The main function is GazeDataReceived that overrides the base class' implementation
     * and checks whether the user stares at one of the controls or not. If the user stares at a control until the
     * dwell time has passed, an action is taken on the control.
     * 
     * Haakon Raudsandmoen and Boerge Roedsjoe, Trondheim, 16. February 2012
     */
    public partial class ControlsTrackbarsForm : BaseForm
    {
        private readonly HomeForm _homeForm;
        private readonly List<Button> _buttons;

        private static TimeSpan DWELL_TIME_CONTROLS_TRACKBARS_FORM = new TimeSpan(0, 0, 0, 1, 500);
        private static TimeSpan DWELL_TIME_TRACKBAR = new TimeSpan(0, 0, 0, 0, 500);

        internal IEyetracker ConnectedTracker { get; set; }
        
        // Constructor that initalizes the layout
        public ControlsTrackbarsForm(HomeForm homeForm)
        {
            InitializeComponent();
            _homeForm = homeForm;
            ConnectedTracker = _homeForm.ConnectedTracker;
            SetTimerInterval(500);

            _buttons = new List<Button> { _homeButton, _controlsPreviousButton , _controlsNextButton};

            UpdateSliderLabel();

            KeyPreview = true;
            KeyDown += ControlsForm_KeyDown;
        }

        private void ControlsForm_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control && e.KeyCode == Keys.Add)
            {
                Log.Write("Trackbars Application: Dwell time: " + DWELL_TIME_TRACKBAR.TotalMilliseconds + " ms");
                DWELL_TIME_CONTROLS_TRACKBARS_FORM = IncreaseDwellTime(DWELL_TIME_CONTROLS_TRACKBARS_FORM);
            }
            if (e.Control && e.KeyCode == Keys.Subtract)
            {
                Log.Write("Trackbars Application: Dwell time: " + DWELL_TIME_TRACKBAR.TotalMilliseconds + " ms");
                DWELL_TIME_CONTROLS_TRACKBARS_FORM = DecreaseDwellTime(DWELL_TIME_CONTROLS_TRACKBARS_FORM);
            }

            UpdateDwellTimeLabel(DWELL_TIME_TRACKBAR);
        }

        // This method override's the base class' implementation, and uses the base class' functions
        // to check if the user is staring at any controls in The Controls Application. If the user
        // stares at a button, a call to DoButtonInteraction(control) is done.
        internal override void GazeDataReceived(object sender, GazeDataEventArgs e)
        {
            base.GazeDataReceived(sender, e);

            var gd = e.GazeDataItem;

            if (GazeDataValid(gd))
            {
                if (IsUserGazingAt(_homeButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _homeButton);
                    DoButtonInteraction(_homeButton, DWELL_TIME_CONTROLS_TRACKBARS_FORM);
                }
                else if (IsUserGazingAt(_controlsNextButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _controlsNextButton);
                    DoButtonInteraction(_controlsNextButton, DWELL_TIME_CONTROLS_TRACKBARS_FORM);
                }
                else if (IsUserGazingAt(_controlsPreviousButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _controlsPreviousButton);
                    DoButtonInteraction(_controlsPreviousButton, DWELL_TIME_CONTROLS_TRACKBARS_FORM);
                }
                else if (IsUserGazingAt(_trackBar1))
                {
                    if (IsUserGazingAtTrackBarValueLessThan(_trackBar1.Value, _trackBar1))
                        _trackBar1.Value -= 1;

                    else if (IsUserGazingAtTrackBarValueHigherThan(_trackBar1.Value, _trackBar1))
                        _trackBar1.Value += 1;

                    UpdateSliderLabel();
                }

                else if (IsUserGazingAt(_trackBar2))
                {
                    _testlabel.Text = "Trackbar 2 value: " + _trackBar2.Value;

                    if (IsUserGazingAtTrackBarValueLessThan(_trackBar2.Value, _trackBar2))
                        _trackBar2.Value -= 1;

                    else if (IsUserGazingAtTrackBarValueHigherThan(_trackBar2.Value, _trackBar2))
                        _trackBar2.Value += 1;
                }

                else if (IsUserGazingAt(_trackBar3))
                {
                    if (IsUserGazingAtTrackBarValueLessThan(_trackBar3.Value, _trackBar3) && UserHasLookedLongEnough(DWELL_TIME_TRACKBAR))
                        _trackBar3.Value -= 1;

                    else if (IsUserGazingAtTrackBarValueHigherThan(_trackBar3.Value, _trackBar3) && UserHasLookedLongEnough(DWELL_TIME_TRACKBAR))
                        _trackBar3.Value += 1;
                }

                else if (IsUserGazingAt(_trackBar4))
                {
                    if (IsUserGazingAtTrackBarValueLessThan(_trackBar4.Value, _trackBar4) && UserHasLookedLongEnough(DWELL_TIME_TRACKBAR))
                        _trackBar4.Value -= 1;

                    else if (IsUserGazingAtTrackBarValueHigherThan(_trackBar4.Value, _trackBar4) && UserHasLookedLongEnough(DWELL_TIME_TRACKBAR))
                        _trackBar4.Value += 1;
                }
                else
                {
                    FirstTimeUserLooksAtControl = true;
                    RemoveHoverEffectFromButtons(_buttons);
                }
            }
        }

        // A helper method to determine if the user is gazing at trackbar-value higher than the
        // value passed as argument
        private bool IsUserGazingAtTrackBarValueHigherThan(int value, TrackBar trackBar)
        {
            // -20 because there is 10 px room at the outsides of the first and last ticks at the trackbar. Need to subtract this.
            var stepWidth = (trackBar.Width - 20) / (trackBar.Maximum - trackBar.Minimum);
            return GazeDot.Location.X + (GazeDot.Width / 2) > trackBar.Parent.PointToScreen(trackBar.Location).X + (value * stepWidth) + (stepWidth / 2);
        }

        // A helper method to determine if the user is gazing at trackbar-value less than the
        // value passed as argument
        private bool IsUserGazingAtTrackBarValueLessThan(int value, TrackBar trackBar)
        {
            // -20 because there is 10 px room at the outsides of the first and last ticks at the trackbar. Need to subtract this.
            var stepWidth = (trackBar.Width - 20) / (trackBar.Maximum - trackBar.Minimum);
            return GazeDot.Location.X + (GazeDot.Width / 2) < trackBar.Parent.PointToScreen(trackBar.Location).X + (value * stepWidth) - (stepWidth / 2);
        }

        // If the trackbar-value is changed we update the correct label with blue color
        private void _trackBar_ValueChanged(object sender, EventArgs e)
        {
            UpdateSliderLabel();
        }

        private void UpdateSliderLabel()
        {
            var text = "";
            switch (_trackBar1.Value)
            {
                case 0:
                    text = "2010";
                    break;
                case 1:
                    text = "2011";
                    break;
                case 2:
                    text = "2012";
                    break;
                case 3:
                    text = "2013";
                    break;
                case 4:
                    text = "2014";
                    break;
                case 5:
                    text = "2015";
                    break;
                case 6:
                    text = "2016";
                    break;
                case 7:
                    text = "2017";
                    break;
                case 8:
                    text = "2018";
                    break;
                case 9:
                    text = "2019";
                    break;
                case 10:
                    text = "2020";
                    break;

            }
            _sliderValueLabel.Text = text;
        }

        // Close the application if home-button is clicked
        private void _homeButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        // Shuts down this form, stops recieving gaze data, shows the main menu and starts
        // recieving gaze data for the main menu
        private void ControlsTrackbarsForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (_homeForm.ConnectedTracker != null)
            {
                _homeForm.ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                _homeForm.ConnectedTracker.GazeDataReceived += _homeForm.GazeDataReceived;
            }
            _homeForm.Show();
        }

        private void ControlsTrackbarsForm_Load(object sender, EventArgs e)
        {
            UpdateDwellTimeLabel(DWELL_TIME_TRACKBAR);
        }
        private void _controlsNextButton_Click(object sender, EventArgs e)
        {
            var controlsButtonsForm = new ControlsButtonsForm(_homeForm);
            if (_homeForm.IsTracking && ConnectedTracker != null)
            {
                ConnectedTracker.GazeDataReceived += controlsButtonsForm.GazeDataReceived;
                ConnectedTracker.GazeDataReceived -= GazeDataReceived;
            }
            controlsButtonsForm.Show();
            Hide();
        }

        private void _controlsPreviousButton_Click(object sender, EventArgs e)
        {
            var controlsCheckboxesForm = new ControlsCheckboxesForm(_homeForm);
            if (_homeForm.IsTracking && ConnectedTracker != null)
            {
                ConnectedTracker.GazeDataReceived += controlsCheckboxesForm.GazeDataReceived;
                ConnectedTracker.GazeDataReceived -= GazeDataReceived;
            }
            controlsCheckboxesForm.Show();
            Hide();
        }
    }
}