﻿using System;
using System.Drawing;
using System.Windows.Forms;
using Tobii.Eyetracking.Sdk;
using Discovery.Properties;

namespace Discovery.DragAndDrop
{
    public partial class DragAndDropForm : BaseForm
    {
        private readonly HomeForm _homeForm;
        private bool _movingLeftObject;
        private bool _movingRightObject;
        private bool _wasGazeDotEnabled;
        private bool _hoverEnabled;
        private TreeNode _nodeInTransit;

        private static TimeSpan PICK_UP_TIME = new TimeSpan(0, 0, 0, 2, 500);
        private static TimeSpan DWELL_TIME_DRAG_AND_DROP_BUTTONS = new TimeSpan(0, 0, 0, 1, 500);

        public DragAndDropForm(HomeForm homeForm)
        {
            InitializeComponent();
            _homeForm = homeForm;
            _exitApplicationButton.Visible = false;
            _treeView1.ExpandAll();
            _treeView2.ExpandAll();

            KeyPreview = true;
            KeyDown += DragAndDrop_KeyDown;
        }

        private void DragAndDrop_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control && e.KeyCode == Keys.Add)
            {
                Log.Write(DWELL_TIME_DRAG_AND_DROP_BUTTONS.TotalMilliseconds + "Drag and Drop Application: Dwell time picture pick up: " + " ms");
                PICK_UP_TIME = IncreaseDwellTime(PICK_UP_TIME);
            }
            if (e.Control && e.KeyCode == Keys.Subtract)
            {
                Log.Write(PICK_UP_TIME.TotalMilliseconds + "Drag and Drop Application: Dwell time picture pick up: " + " ms");
                PICK_UP_TIME = DecreaseDwellTime(PICK_UP_TIME);
            }

            UpdateDwellTimeLabel(PICK_UP_TIME);
        }

        private void _homeButton_Click(object sender, EventArgs e)
        {
            Close();
        }
        private void _enableHoverButton_CheckedChanged(object sender, EventArgs e)
        {
            if (!_hoverEnabled)
            {
                _hoverEnabled = true;
                _enableHoverButton.Text = "Hover Effect is" + Environment.NewLine + "ON";
                GazeDot.Visible = false;
            }
            else
            {
                _hoverEnabled = false;
                _enableHoverButton.Text = "Hover Effect is" + Environment.NewLine + "OFF";
                GazeDot.Visible = true;
            }
        }

        // Runs every time we get data from the eyetracker.
        internal override void GazeDataReceived(object sender, GazeDataEventArgs e)
        {
            base.GazeDataReceived(sender, e);

            var gd = e.GazeDataItem;

            if (GazeDataValid(gd))
            {
                //Left tree
                //Checks the "root node"
                if (IsUserGazingAtNode(_treeView1.Nodes[0]) && _movingRightObject)
                {
                    if (UserHasLookedLongEnoughAtNode(PICK_UP_TIME, _treeView1.Nodes[0]))
                        ReleaseFromGazeDot(null, _nodeInTransit, _treeView1);
                }
                //Checks the other nodes
                foreach (TreeNode node in _treeView1.Nodes[0].Nodes)
                {
                    if (IsUserGazingAtNode(node))
                    {
                        if (_hoverEnabled)
                            node.BackColor = Color.LightGray;
                        if (UserHasLookedLongEnoughAtNode(PICK_UP_TIME, node))
                        {

                            if (_movingRightObject)
                                ReleaseFromGazeDot(node, _nodeInTransit);
                            else if (_movingLeftObject)
                                AttachToGazeDot(node, 1);
                            else
                                AttachToGazeDot(node, 1);
                        }
                    }
                    else
                        node.BackColor = Color.White;
                }
                //Rigth tree
                //Checks the "root node"
                if (IsUserGazingAtNode(_treeView2.Nodes[0]) && _movingLeftObject)
                {
                    if (UserHasLookedLongEnoughAtNode(PICK_UP_TIME, _treeView2.Nodes[0]))
                        ReleaseFromGazeDot(null, _nodeInTransit, _treeView2);
                }
                //Checks the other nodes
                foreach (TreeNode node in _treeView2.Nodes[0].Nodes)
                {
                    if (IsUserGazingAtNode(node))
                    {
                        if (_hoverEnabled)
                            node.BackColor = Color.LightGray;
                        if (UserHasLookedLongEnoughAtNode(PICK_UP_TIME, node))
                        {
                            if (_movingLeftObject)
                                ReleaseFromGazeDot(node, _nodeInTransit);
                            else if (_movingRightObject)
                                AttachToGazeDot(node, 2);
                            else
                                AttachToGazeDot(node, 2);
                        }
                    }
                    else
                        node.BackColor = Color.White;
                }

                //Checks the meny buttons
                if (IsUserGazingAt(_homeButton))
                    DoButtonInteraction(_homeButton, DWELL_TIME_DRAG_AND_DROP_BUTTONS);

                else if (IsUserGazingAt(_enableHoverButton))
                {
                    _enableHoverButton.BackgroundImage = Resources.eye_hover;
                    if (UserHasLookedLongEnough())
                        _enableHoverButton.Checked = !_enableHoverButton.Checked;
                }
                else
                {
                    FirstTimeUserLooksAtControl = true;
                    _homeButton.BackgroundImage = null;
                    _enableHoverButton.BackgroundImage = null;
                }
            }
        }

        //This code initiates a move operation on the item that is being dragged.
        private void _treeView_ItemDrag(object sender, ItemDragEventArgs e)
        {
            DoDragDrop(e.Item, DragDropEffects.Move);
        }

        private void _treeView_DragEnter(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.Move;
        }
        /*
        The code also does not handle the scenario in which a user does not drop the object 
         * on a particular node in the destination TreeView control. To handle this scenario, 
         * test whether the DestinationNode is null; if it is, you can add the object that is 
         * being dragged to the root of the TreeView control, and so on.
         * 
         * http://support.microsoft.com/kb/307968
        */
        private void _treeView_DragDrop(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent("System.Windows.Forms.TreeNode", false))
            {
                TreeNode movedNode;

                var pt = ((TreeView)sender).PointToClient(new Point(e.X, e.Y));
                var destinationNode = ((TreeView)sender).GetNodeAt(pt);
                movedNode = (TreeNode)e.Data.GetData("System.Windows.Forms.TreeNode");

                if (destinationNode.TreeView != movedNode.TreeView)
                {
                    destinationNode.Nodes.Add((TreeNode)movedNode.Clone());
                    destinationNode.Expand();

                    //Remove Original Node
                    movedNode.Remove();
                }
            }
        }
        //Checks if a user is gazing at a node (or node icon).
        internal bool IsUserGazingAtNode(TreeNode node)
        {
            //TreeNode node = _treeView1.Nodes[0].Nodes[nodeNumber];
            return (GazeDot.Location.X) + GazeDot.Width / 2 > node.TreeView.PointToScreen(node.Bounds.Location).X - 200 &&
                            (GazeDot.Location.X) + GazeDot.Width / 2 < node.TreeView.PointToScreen(node.Bounds.Location).X + node.Bounds.Width &&
                            (GazeDot.Location.Y) + GazeDot.Height / 2 > node.TreeView.PointToScreen(node.Bounds.Location).Y &&
                            (GazeDot.Location.Y) + GazeDot.Height / 2 < node.TreeView.PointToScreen(node.Bounds.Location).Y + node.Bounds.Height;
        }

        //Attaches the node to the GazeDot
        private void AttachToGazeDot(TreeNode node, int tree)
        {
            //ImageIndex of node[0] is -1
            var temp = 0;
            if (node.ImageIndex == -1)
            {
                temp = 1;
            }
            //Decides origin tree
            TreeView treeView;
            if (tree == 1)
            {
                treeView = _treeView1;
                _movingLeftObject = true;
            }
            else
            {
                treeView = _treeView2;
                _movingRightObject = true;
            }

            //Resize and set the backgroundImage of the Gazedot
            GazeDot.Size = new Size(100, 75);
            GazeDot.BackgroundImage = treeView.ImageList.Images[node.ImageIndex + temp];
            GazeDot.Visible = true;

            _nodeInTransit = node;

        }
        //Releases the node from the GazeDot to a destination node or tree
        private void ReleaseFromGazeDot(TreeNode DestinationNode, TreeNode NodeInTransit, TreeView DestinationTree = null)
        {
            //Remove from parent tree
            NodeInTransit.TreeView.Nodes[0].Nodes.Remove(NodeInTransit);
            if (DestinationTree != null)
            {
                //Add to destination tree
                DestinationTree.Nodes[0].Nodes.Add(NodeInTransit);
            }
            else
            {
                //Add to destination node
                DestinationNode.TreeView.Nodes[0].Nodes.Insert(DestinationNode.Index + 1, NodeInTransit);

            }
            _movingRightObject = false;
            _movingLeftObject = false;

            //Resizes and removes the backgroundImage of the Gazedot
            GazeDot.Size = GAZE_DOT_SIZE;
            GazeDot.BackgroundImage = null;
            if (_hoverEnabled)
                GazeDot.Visible = false;
            else
                GazeDot.Visible = true;
        }
        private void DragAndDropForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (_homeForm.ConnectedTracker != null)
            {
                _homeForm.ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                _homeForm.ConnectedTracker.GazeDataReceived += _homeForm.GazeDataReceived;
            }
            //Resizes and removes the backgroundImage of the Gazedot
            GazeDot.Size = GAZE_DOT_SIZE;
            GazeDot.BackgroundImage = null;

            //Turn off/on Gazedot according to its initial state
            if (!_wasGazeDotEnabled)
                GazeDot.Visible = false;
            _homeForm.Show();
        }

        private void DragAndDropForm_Load(object sender, EventArgs e)
        {
            _enableHoverButton.Text = "Hover Effect is" + Environment.NewLine + "OFF";
            if (!GazeDot.Visible)
            {
                GazeDot.Visible = true;
                GazeDot.Size = GAZE_DOT_SIZE;
                _wasGazeDotEnabled = false;
            }
            else
                _wasGazeDotEnabled = true;
            UpdateDwellTimeLabel(PICK_UP_TIME);
        }
    }
}
