﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using Discovery.Browser;
using Discovery.Calib;
using Discovery.Controls;
using Discovery.DragAndDrop;
using Discovery.PictureGallery;
using Discovery.TextReading;
using Discovery.UFO;
using Tobii.Eyetracking.Sdk;
using Tobii.Eyetracking.Sdk.Exceptions;
using Tobii.Eyetracking.Sdk.Time;

namespace Discovery
{
    public partial class HomeForm : BaseForm
    {
        private CalibrationForm _calibrationForm;
        private ControlsButtonsForm _controlsButtonsForm;
        private PictureGalleryForm _pictureGalleryForm;
        private UFOForm _ufoForm;
        private WebBrowserForm _browserForm;
        private TextReadingForm _textReadingForm;
        private DragAndDropForm _dragAndDropForm;

        private readonly List<Button> _buttons;

        private SyncManager _syncManager;
        private readonly Clock _clock;

        private static TimeSpan DWELL_TIME_HOME_BUTTONS = new TimeSpan(0, 0, 0, 1, 500);

        internal EyetrackerBrowser TrackerBrowser { get; set; }
        internal IEyetracker ConnectedTracker { get; set; }
        internal bool IsTracking { get; set; }
        internal bool FirstTimeUserSeesTheCalibrationInstructions { get; set; }
        internal bool FirstTimeUserSeesThePictureGalleryInstructions { get; set; }
        internal string ConnectionName { get; set; }

        public HomeForm()
        {
            InitializeComponent();

            _homeButton.Visible = false;
            _exitApplicationButton.Visible = true;
            FirstTimeUserSeesTheCalibrationInstructions = true;
            FirstTimeUserSeesThePictureGalleryInstructions = true;

            // To enable the scroll bar and make the panel scrollable
            _splitContainer.Panel2.AutoScrollMinSize = _splitContainer.Panel2.GetPreferredSize(new Size(1, 1));

            _buttons = new List<Button>{
                _calibrationButton,
                _controlsButton,
                _dragAndDropButton,
                _pictureGalleryButton,
                _browserButton,
                _ufoButton,
                _aboutButton,
                _exitApplicationButton};

            _clock = new Clock();

            TrackerBrowser = new EyetrackerBrowser();
            TrackerBrowser.EyetrackerFound += EyetrackerFound;

            KeyPreview = true;
            KeyDown += HomeForm_KeyDown;
        }

        private void HomeForm_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control && e.KeyCode == Keys.Add)
            {
                Log.Write("Main Menu: Dwell time buttons: " + DWELL_TIME_HOME_BUTTONS.TotalMilliseconds);
                DWELL_TIME_HOME_BUTTONS = IncreaseDwellTime(DWELL_TIME_HOME_BUTTONS);
            }
            if (e.Control && e.KeyCode == Keys.Subtract)
            {
                Log.Write("Main Menu: Dwell time buttons: " + DWELL_TIME_HOME_BUTTONS.TotalMilliseconds);
                DWELL_TIME_HOME_BUTTONS = DecreaseDwellTime(DWELL_TIME_HOME_BUTTONS);
            }

            UpdateDwellTimeLabel(DWELL_TIME_HOME_BUTTONS);
        }

        private void HomeForm_Load(object sender, EventArgs e)
        {
            if (ConnectedTracker == null)
            {
                // Start browsing for eyetrackers on the network
                // When an eye tracker is found, the EyeTrackerFound method will be called
                TrackerBrowser.Start();
            }
            GazeDot.Size = GAZE_DOT_SIZE;
            UpdateDwellTimeLabel(DWELL_TIME_HOME_BUTTONS);
        }
        private void HomeForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            Log.Write("*** Discovery Shutting down ***");

            // Shutdown browser service
            TrackerBrowser.Stop();

            // Cleanup connections
            DisconnectTracker();
        }

        private void EyetrackerFound(object sender, EyetrackerInfoEventArgs e)
        {
            // Disconnect existing connection
            DisconnectTracker();

            // Trying to connect to the eye tracker
            ConnectToTracker(e.EyetrackerInfo);

            // Start tracking (subscribing to gaze data stream)
            StartTracking();
        }
        private void ConnectToTracker(EyetrackerInfo info)
        {
            try
            {
                ConnectedTracker = EyetrackerFactory.CreateEyetracker(info);
                ConnectedTracker.ConnectionError += HandleConnectionError;

                _syncManager = new SyncManager(_clock, info);

                ConnectedTracker.GazeDataReceived += GazeDataReceived;
                ConnectionName = info.ProductId;
            }
            catch (EyetrackerException ee)
            {
                if (ee.ErrorCode == 0x20000402)
                {
                    MessageBox.Show("Failed to upgrade protocol. " +
                        "This probably means that the firmware needs" +
                        " to be upgraded to a version that supports the new sdk.", "Upgrade Failed", MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
                }
                else
                {
                    MessageBox.Show("Eyetracker responded with error " + ee, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                DisconnectTracker();
            }
            catch (Exception)
            {
                MessageBox.Show("Could not connect to eyetracker. Try to open close and open the Calibration window again", "Connection Failed", MessageBoxButtons.OK, MessageBoxIcon.Error);
                DisconnectTracker();
            }
        }
        private void StartTracking()
        {
            if (IsTracking)
            {
                // Unsubscribe from gaze data stream
                ConnectedTracker.StopTracking();
                IsTracking = false;
            }
            else
            {
                // Start subscribing to gaze data stream
                ConnectedTracker.StartTracking();
                IsTracking = true;
            }
        }
        private void HandleConnectionError(object sender, ConnectionErrorEventArgs e)
        {
            DisconnectTracker();
        }
        internal void DisconnectTracker()
        {
            if (ConnectedTracker != null)
            {
                ConnectedTracker.GazeDataReceived -= GazeDataReceived;

                if (_calibrationForm != null)
                    ConnectedTracker.GazeDataReceived -= _calibrationForm.GazeDataReceived;
                if (_controlsButtonsForm != null)
                    ConnectedTracker.GazeDataReceived -= _controlsButtonsForm.GazeDataReceived;
                if (_pictureGalleryForm != null)
                    ConnectedTracker.GazeDataReceived -= _pictureGalleryForm.GazeDataReceived;
                if (_ufoForm != null)
                    ConnectedTracker.GazeDataReceived -= _ufoForm.GazeDataReceived;
                if (_browserForm != null)
                    ConnectedTracker.GazeDataReceived -= _browserForm.GazeDataReceived;
                if (_textReadingForm != null)
                    ConnectedTracker.GazeDataReceived -= _textReadingForm.GazeDataReceived;
                if (_dragAndDropForm != null)
                    ConnectedTracker.GazeDataReceived -= _dragAndDropForm.GazeDataReceived;

                ConnectedTracker.Dispose();
                ConnectedTracker = null;
                IsTracking = false;

                _syncManager.Dispose();
            }
        }
        internal override void GazeDataReceived(object sender, GazeDataEventArgs e)
        {
            base.GazeDataReceived(sender, e);

            var gd = e.GazeDataItem;

            if (GazeDataValid(gd))
            {
                if (UserWantsToScroll(_splitContainer.Panel2, "down"))
                    _splitContainer.Panel2.VerticalScroll.Value += 3;

                else if (UserWantsToScroll(_splitContainer.Panel2, "up"))
                {
                    if (_splitContainer.Panel2.VerticalScroll.Value >= 3)
                        _splitContainer.Panel2.VerticalScroll.Value -= 3;
                }
                else if (IsUserGazingAt(_aboutButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _aboutButton);
                    DoButtonInteraction(_aboutButton, DWELL_TIME_HOME_BUTTONS);
                }

                else if (IsUserGazingAt(_calibrationButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _calibrationButton);
                    DoButtonInteraction(_calibrationButton, DWELL_TIME_HOME_BUTTONS);
                }
                    
                else if (IsUserGazingAt(_controlsButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _controlsButton);
                    DoButtonInteraction(_controlsButton, DWELL_TIME_HOME_BUTTONS);
                }

                else if (IsUserGazingAt(_dragAndDropButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _dragAndDropButton);
                    DoButtonInteraction(_dragAndDropButton, DWELL_TIME_HOME_BUTTONS);
                }

                else if (IsUserGazingAt(_pictureGalleryButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _pictureGalleryButton);
                    DoButtonInteraction(_pictureGalleryButton, DWELL_TIME_HOME_BUTTONS);
                }

                else if (IsUserGazingAt(_browserButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _browserButton);
                    DoButtonInteraction(_browserButton, DWELL_TIME_HOME_BUTTONS);
                }

                else if (IsUserGazingAt(_ufoButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _ufoButton);
                    DoButtonInteraction(_ufoButton, DWELL_TIME_HOME_BUTTONS);
                }
                    
                else if (IsUserGazingAt(_exitApplicationButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _exitApplicationButton);
                    DoButtonInteraction(_exitApplicationButton, DWELL_TIME_HOME_BUTTONS);
                }
                    
                else
                {
                    FirstTimeUserLooksAtControl = true;
                    RemoveHoverEffectFromButtons(_buttons);
                }
            }
        }
        #region ButtonClicks
        private void _aboutButton_Click(object sender, EventArgs e)
        {
            var aboutForm = new AboutForm();
            if (ConnectedTracker != null)
            {
                ConnectedTracker.GazeDataReceived += aboutForm.GazeDataReceived;
                ConnectedTracker.GazeDataReceived -= GazeDataReceived;
            }

            aboutForm.ShowDialog();

            if (ConnectedTracker != null)
            {
                ConnectedTracker.GazeDataReceived -= aboutForm.GazeDataReceived;
                ConnectedTracker.GazeDataReceived += GazeDataReceived;
            }
        }
        private void _calibrationButton_Click(object sender, EventArgs e)
        {
            _calibrationForm = new CalibrationForm(this);
            if (ConnectedTracker != null)
            {
                ConnectedTracker.GazeDataReceived += _calibrationForm.GazeDataReceived;
                ConnectedTracker.GazeDataReceived -= GazeDataReceived;
            }
            _calibrationForm.Show();
            Hide();
        }
        private void _controlsButton_Click(object sender, EventArgs e)
        {
            _controlsButtonsForm = new ControlsButtonsForm(this);
            if (IsTracking && ConnectedTracker != null)
            {
                ConnectedTracker.GazeDataReceived += _controlsButtonsForm.GazeDataReceived;
                ConnectedTracker.GazeDataReceived -= GazeDataReceived;
            }
            _controlsButtonsForm.Show();
            Hide();
        }

        private void _dragAndDropButton_Click(object sender, EventArgs e)
        {
            _dragAndDropForm = new DragAndDropForm(this);
            if (IsTracking && ConnectedTracker != null)
            {
                ConnectedTracker.GazeDataReceived += _dragAndDropForm.GazeDataReceived;
                ConnectedTracker.GazeDataReceived -= GazeDataReceived;
            }
            _dragAndDropForm.Show();
            Hide();
        }

        private void _pictureGalleryButton_Click(object sender, EventArgs e)
        {
            _pictureGalleryForm = new PictureGalleryForm(this);
            if (IsTracking && ConnectedTracker != null)
            {
                ConnectedTracker.GazeDataReceived += _pictureGalleryForm.GazeDataReceived;
                ConnectedTracker.GazeDataReceived -= GazeDataReceived;
            }
            _pictureGalleryForm.Show();
            Hide();

            if (FirstTimeUserSeesThePictureGalleryInstructions)
            {
                var pictureGalleryInstructionsForm = new PictureGalleryInstructionsForm(_pictureGalleryForm);

                if (ConnectedTracker != null)
                {
                    ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                    ConnectedTracker.GazeDataReceived += pictureGalleryInstructionsForm.GazeDataReceived;
                }

                var dialogResult = pictureGalleryInstructionsForm.ShowDialog();

                if (dialogResult != DialogResult.OK)
                {
                    if (ConnectedTracker != null)
                        ConnectedTracker.GazeDataReceived += GazeDataReceived;
                    return;
                }

                FirstTimeUserSeesThePictureGalleryInstructions = false;
            }
        }

        private void _browserButton_Click(object sender, EventArgs e)
        {
            _browserForm = new WebBrowserForm(this);
            if (IsTracking && ConnectedTracker != null)
            {
                ConnectedTracker.GazeDataReceived += _browserForm.GazeDataReceived;
                ConnectedTracker.GazeDataReceived -= GazeDataReceived;
            }
            _browserForm.Show();
            Hide();
        }

        private void _ufoButton_Click(object sender, EventArgs e)
        {
            _ufoForm = new UFOForm(this);
            if (IsTracking && ConnectedTracker != null)
            {
                ConnectedTracker.GazeDataReceived += _ufoForm.GazeDataReceived;
                ConnectedTracker.GazeDataReceived -= GazeDataReceived;
            }
            _ufoForm.Show();
            Hide();
        }
        // Originally created a text reading application, but this has been removed lately
        // Keeps the code here if one wants to enable this application again
        private void _textReadingButton_Click(object sender, EventArgs e)
        {
            _textReadingForm = new TextReadingForm(this);
            if (IsTracking && ConnectedTracker != null)
            {
                ConnectedTracker.GazeDataReceived += _textReadingForm.GazeDataReceived;
                ConnectedTracker.GazeDataReceived -= GazeDataReceived;
            }
            _textReadingForm.Show();
            Hide();
        }

        private void _exitApplicationButton_Click(object sender, EventArgs e)
        {
            var quitForm = new QuitForm();
            if (ConnectedTracker != null)
            {
                ConnectedTracker.GazeDataReceived += quitForm.GazeDataReceived;
                ConnectedTracker.GazeDataReceived -= GazeDataReceived;
            }

            var userWantsToQuit = quitForm.ShowDialog();
            
            if (ConnectedTracker != null)
                ConnectedTracker.GazeDataReceived -= quitForm.GazeDataReceived;

            if (userWantsToQuit == DialogResult.Yes)
            {
                if (ConnectedTracker != null)
                    ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                DisconnectTracker();
                Close();
            }
            else if (ConnectedTracker != null)
                ConnectedTracker.GazeDataReceived += GazeDataReceived;
        }
        #endregion
    }
}
